rem $DBA/grantall.sql
rem
rem Creates a script to grant ALL object privileges on all non "System" objects
rem to the specified "Oracle" account
rem
rem Input parameters:
rem
rem	&&OracleUser = Oracle username (Example:  OPS$ORACLE)
rem
rem Last Change 06/25/96 by Brian Lomasky
rem
create table grantall_temp (lineno number, gtxt varchar2(255));

set feedback off
set heading off 
set term on
set echo off
set pagesize 0
set newpage 0
set linesize 255

declare
	x_owner		varchar2(30);
	x_object_name	varchar2(128);
	linen		number;
	txt		varchar2(255);
	p_owner		varchar2(30);
	p_pass		varchar2(30);
	llen		number;
cursor c1 is
	select owner, object_name
	from sys.dba_objects
	where owner <> 'SYS' and owner <> 'SYSTEM' and OWNER <> 'PUBLIC' and
		owner <> '&&OracleUser' and object_type <> 'PACKAGE BODY' and
		object_type <> 'CLUSTER' and object_type <> 'SYNONYM' and
		object_type <> 'INDEX' and object_type <> 'DATABASE LINK' and
		object_type <> 'TRIGGER'
	order by 1;
begin
	linen := 1;
	llen := 0;
	p_owner := '@';
	open c1;
	loop
		fetch c1 into x_owner, x_object_name;
		exit when c1%notfound;
		if x_owner <> p_owner then
			if p_owner <> '@' then
				txt := 'connect /;';
				linen := linen + 1;
				insert into grantall_temp values
					(linen, txt);
				txt := 'alter user '||rtrim(p_owner)||
					' identified by values '||''''||p_pass
					||''''||';';
				if length(txt) > llen then
					llen := length(txt);
				end if;
				linen := linen + 1;
				insert into grantall_temp values (
					linen, txt);
			end if;
			declare
				missing_user	exception;
			cursor c2 (c_usr varchar2) is
				select password
				from dba_users
				where username = c_usr;
			begin
				open c2 (x_owner);
				fetch c2 into p_pass;
				if c2%notfound then
					raise missing_user;
				end if;
				close c2;
				commit;
			end;
			txt := 'alter user '||rtrim(x_owner)||
				' identified by xyzzy;';
			if length(txt) > llen then
				llen := length(txt);
			end if;
			linen := linen + 1;
			insert into grantall_temp values (linen, txt);
			txt := 'prompt Processing objects owned by '||
				rtrim(x_owner)||'...';
			linen := linen + 1;
			insert into grantall_temp values (linen, txt);
			txt := 'connect '||rtrim(x_owner)||'/xyzzy;';
			linen := linen + 1;
			insert into grantall_temp values (linen, txt);
			p_owner := x_owner;
		end if;
		txt := 'grant all on '||rtrim(x_owner)||'.'||
			rtrim(x_object_name);
		if length(txt) > llen then
			llen := length(txt);
		end if;
		linen := linen + 1;
		insert into grantall_temp values (linen, txt);
		txt := '   to &&OracleUser with grant option;';
		linen := linen + 1;
		insert into grantall_temp values (linen, txt);
	end loop;
	close c1;
	if p_owner <> '@' then
		txt := 'connect /;';
		linen := linen + 1;
		insert into grantall_temp values (linen, txt);
		txt := 'alter user '||rtrim(p_owner)||
			' identified by values '||''''||p_pass||''''||';';
		if length(txt) > llen then
			llen := length(txt);
		end if;
		linen := linen + 1;
		insert into grantall_temp values (linen, txt);
	end if;
	txt := 'set linesize '||to_char(llen);
	linen := 0;
	insert into grantall_temp values (linen, txt);
	commit;
end;
/
set term off
set linesize 80
spool grall_tmp.sql
select rtrim(gtxt) from grantall_temp where lineno = 0;
spool off
spool grall.sql
@grall_tmp
select rtrim(gtxt) from grantall_temp where lineno > 0 order by lineno;
spool off
drop table grantall_temp;
set term on
host rm grall_tmp.sql
@grall
host rm grall.sql
prompt
prompt Done - &&OracleUser has been granted all privileges
prompt        for all non-system objects.
prompt
prompt Note:  Errors pertaining to "ORA-01720: grant option does not exist for"
prompt        indicate that the displayed owner does not have the "WITH GRANT
prompt        OPTION" to the displayed object, for ALL privileges.  You can
prompt        manually grant all access to that object and then rerun this
prompt        procedure.
prompt
exit
