rem $DBA/readonly.sql
rem
rem Script to grant SELECT access to each non-SYS/non-SYSTEM-owned table, view,
rem and snapshot to the READONLY role.
rem
rem This script can be used to eliminate the need to grant the SELECT ANY TABLE
rem privilege to users.
rem
rem It should be executed whenever a new table, view, or snapshot is created
rem (or on a periodic basis, such as a nightly batch job to ensure that any new
rem tables, views, or snapshots created that day are found).
rem
rem Restrictions:
rem   1) Before running this script, the DBA must create the READONLY role and
rem	 grant it CONNECT privilege.  Example:
rem		create role readonly;
rem		grant connect to readonly;
rem   2) There must be an existing OPS$username account, having DBA privileges.
rem      Example:
rem		create user ops$oracle identified externally
rem			temporary tablespace temp;
rem		grant connect, resource, dba to ops$oracle;
rem
rem <<<<<<<<<< MODIFICATION HISTORY >>>>>>>>>>
rem   Date	Programmer	Description
rem --------	---------------	----------------------------------------------
rem 06/03/97	Brian Lomasky	Include views and snapshots
rem 03/28/97	Brian Lomasky	Original
rem
whenever sqlerror exit failure
set echo off
set feedback off
set heading off 
set linesize 80
set newpage 0
set pagesize 0
set term on
create table readonly_temp (lineno number, gtxt varchar2(80));
declare
	my_username	sys.dba_tables.owner%TYPE;
	x_owner		sys.dba_tables.owner%TYPE;
	p_owner		sys.dba_tables.owner%TYPE;
	x_table_name	sys.dba_tables.table_name%TYPE;
	linen		number;
	txt		varchar2(80);
	p_pass		varchar2(30);
	cursor c1 (me in varchar2) is
		select owner, table_name
		from dba_tables
		where owner <> 'SYS' and owner <> 'SYSTEM' and owner <> me
		union
		select owner, view_name
		from dba_views
		where owner <> 'SYS' and owner <> 'SYSTEM' and owner <> me
		union
		select owner, name
		from dba_snapshots
		where owner <> 'SYS' and owner <> 'SYSTEM' and owner <> me
		order by 1;
begin
	select user into my_username from dual;
	linen := 1;
	p_owner := '@';
	txt := 'set echo on';
	linen := linen + 1;
	insert into readonly_temp values (linen, txt);
	txt := 'set feedback off';
	linen := linen + 1;
	insert into readonly_temp values (linen, txt);
	txt := 'set term on';
	linen := linen + 1;
	insert into readonly_temp values (linen, txt);
	txt := 'set verify off';
	linen := linen + 1;
	insert into readonly_temp values (linen, txt);
	open c1 (my_username);
	loop
		fetch c1 into x_owner, x_table_name;
		exit when c1%notfound;
		if x_owner <> p_owner then
			if p_owner <> '@' then
				txt := 'set term off';
				linen := linen + 1;
				insert into readonly_temp values (linen, txt);
				txt := 'connect /;';
				linen := linen + 1;
				insert into readonly_temp values (linen, txt);
				txt := 'alter user '||rtrim(p_owner)||
					' identified by values '||''''||p_pass
					||''''||';';
				linen := linen + 1;
				insert into readonly_temp values (linen, txt);
				txt := 'set term on';
				linen := linen + 1;
				insert into readonly_temp values (linen, txt);
			end if;
			declare
				missing_user	exception;
			cursor c2 (c_usr varchar2) is
				select password
				from dba_users
				where username = c_usr;
			begin
				open c2 (x_owner);
				fetch c2 into p_pass;
				if c2%notfound then
					raise missing_user;
				end if;
				close c2;
				commit;
			end;
			txt := 'set term off';
			linen := linen + 1;
			insert into readonly_temp values (linen, txt);
			txt := 'alter user '||rtrim(x_owner)||
				' identified by xyzzy;';
			linen := linen + 1;
			insert into readonly_temp values (linen, txt);
			txt := 'set term on';
			linen := linen + 1;
			insert into readonly_temp values (linen, txt);
			txt := 'prompt Processing objects owned by '||
				rtrim(x_owner)||'...';
			linen := linen + 1;
			insert into readonly_temp values (linen, txt);
			txt := 'set term off';
			linen := linen + 1;
			insert into readonly_temp values (linen, txt);
			txt := 'connect '||rtrim(x_owner)||'/xyzzy;';
			linen := linen + 1;
			insert into readonly_temp values (linen, txt);
			txt := 'set term on';
			linen := linen + 1;
			insert into readonly_temp values (linen, txt);
			p_owner := x_owner;
		end if;
		txt := 'grant SELECT on ' || rtrim(x_owner) || '.' ||
			rtrim(x_table_name) || ' to READONLY;';
		linen := linen + 1;
		insert into readonly_temp values (linen, txt);
	end loop;
	close c1;
	if p_owner <> '@' then
		txt := 'set term off';
		linen := linen + 1;
		insert into readonly_temp values (linen, txt);
		txt := 'connect /;';
		linen := linen + 1;
		insert into readonly_temp values (linen, txt);
		txt := 'alter user '||rtrim(p_owner)||
			' identified by values '||''''||p_pass||''''||';';
		linen := linen + 1;
		insert into readonly_temp values (linen, txt);
		txt := 'set term on';
		linen := linen + 1;
		insert into readonly_temp values (linen, txt);
	end if;
	commit;
end;
/
set term off
spool readonly_tmp.sql
select rtrim(gtxt) from readonly_temp order by lineno;
spool off
drop table readonly_temp;
@readonly_tmp
host rm readonly_tmp.sql
prompt
prompt . Done - The READONLY role has been granted SELECT privilege for all
prompt .        non-sys/non-system/non-OPS$ tables/views/snapshots.
prompt
exit
