rem $TOOLS/autoplan.sql
rem
rem Haven't you wondered about performance of the statement that you just wrote
rem in SQL*Plus.  Most often you might have thought of using Explain plan.  But
rem you can't remember the syntax of explain plan or you could not remember that
rem select statement to show the plan from plan table.
rem
rem The following script automates the task of EXPLAIN PLAN.
rem Note that the SQL statement is intact at the end.
rem
rem Restrictions:
rem	1) The user needs DBA privileges or SELECT access to V_$SESSION.
rem	   Example:  As user SYS, type:  GRANT SELECT ON V_$SESSION TO PUBLIC;
rem	2) This autoplan.sql script must reside in the current directory, or
rem	   anywhere in the ORACLE_PATH environment variable.  (This will allow
rem	   easy execution by just typing:  @autoplan
rem	   at any SQL*Plus prompt).
rem     3) The PLAN_TABLE must already exist.  To create the PLAN_TABLE, run
rem        the following SQL*Plus command from the user's account:
rem		@$ORACLE_HOME/rdbms/admin/utlxplan  
rem
rem <<<<<<<<<< MODIFICATION HISTORY >>>>>>>>>>
rem 04/22/97	Brian Lomasky	Support Oracle 7.3 version of plan_table (COST,
rem				  BYTES, and CARDINALITY are only available in
rem				  Oracle 7.3).  Create unique statement id from
rem				  user, sid, and date.  Remove any semicolon.
rem 1997			Original script from Compuserve Oracle forum,
rem				  74053,36, Ramesh Meda
rem
set echo off
set feedback off
set linesize 80
set termout off
set verify off
save autotemp replace
column myids new_value myid noprint
select user || to_char(sysdate, 'YYMMDD') || sid myids from v$session;
delete from plan_table where statement_id = '&myid';
get autotemp nolist
ch /;//
1
ch //explain plan set statement_id='&myid' for /
set termout on
/
set pagesize 9999
column lvl format a12 heading "Level"
column operation format a17 heading "Operation"
column obj format a24 heading "Object"
column object_instance format 99 heading "Obj|Ins"
column opt format a4 heading "Opt|Plan"
column cost format 9999 heading "Cost"
column cardinality format a4 heading " #|rows"
column bytes format a4 heading " #|Byte"
spool autoplan.lis
select lpad(' ',level-1) || level || '.' || decode(id,0,0,nvl(position,0)) lvl,
	decode(operation,'TABLE ACCESS','TBL ACCESS','BITMAP CONVERSION',
		'BITMAP CONV','BITMAP INDEX','BITMAP INX',operation) || ' ' ||
		decode(options,'BY ROWID','ROWID','CARTESIAN','CART',
		'TO ROWIDS','ROWID','SINGLE VALUE','SNGL',options) operation,
	decode(object_owner,null,'',
		decode(sign(23-length(rtrim(object_owner)||object_name)),
			-1,object_name,rtrim(object_owner)||'.'||object_name)
		) obj,
	object_instance,
	decode(optimizer,'CHOOSE','Choo','ANALYZED','Anal',optimizer) opt,
	decode(nvl(cost,0),0,'',
		decode(sign(cost-1048576),-1,
			decode(sign(cost-9999),-1,
				substr(to_char(cost, '9999'),2,99),
				substr(to_char(trunc(cost/1024),'999'), 2,99)
					|| 'K'
				),
			substr(to_char(trunc(cost/1048576),'999'),2,99) || 'M')
		) cost,
	decode(nvl(cardinality,0),0,'',
		decode(sign(cardinality-1048576),-1,
			decode(sign(cardinality-9999),-1,
				substr(to_char(cardinality, '9999'),2,99),
				substr(to_char(trunc(cardinality/1024),'999'),
					2,99) || 'K'
				),
			substr(to_char(trunc(cardinality/1048576),'999'),2,99)
				|| 'M'
			)
		) cardinality,
	decode(nvl(bytes,0),0,'',
		decode(sign(bytes-1048576),-1,
			decode(sign(bytes-9999),-1,
				substr(to_char(bytes, '9999'),2,99),
				substr(to_char(trunc(bytes/1024),'999'),2,99)
					|| 'K'
				),
			substr(to_char(trunc(bytes/1048576),'999'),2,99) || 'M'
			)
		) bytes
	from plan_table 
	start with id = 0 and statement_id = '&myid'
	connect by prior id = parent_id and statement_id = '&myid';
spool off
prompt
prompt Created autoplan.lis explain plan report file...
prompt
get autotemp nolist
