rem $DBA/crlinks.sql
rem
rem This script generates another script that will include all the CREATE
rem DATABASE LINK statements for those in the database.
rem
rem This script must be run by a user with the DBA role under Oracle7.
rem
rem (The SYS.LINK$ table is used, since the password column is not visible
rem  in the DBA_DB_LINKS view).
rem
rem Last Change 05/05/97 by Brian Lomasky
rem
set verify off
set feedback off
set echo off
set pagesize 0
set serveroutput on size 100000
set termout on
define q=chr(39)
select 'Creating database link build script...' from dual;

create table l_temp (	lineno NUMBER,
			text VARCHAR2(80));

declare
	cursor link_cursor is select
		u.name,
		l.name,
		l.userid,
		l.password,
		l.host
		from sys.link$ l, sys.user$ u
		where l.owner# = u.user#
		order by u.name, l.name;

	cursor pwd_cursor (c_user varchar2) is
		select password from sys.dba_users where username = c_user;

	alter_user		sys.user$.name%TYPE;
	prev_owner		sys.user$.name%TYPE;
	lv_owner		sys.user$.name%TYPE;
	lv_db_link		sys.link$.name%TYPE;
	lv_username		sys.link$.userid%TYPE;
	lv_password		sys.link$.password%TYPE;
	lv_host			sys.link$.host%TYPE;
	user_password		sys.dba_users.password%TYPE;
	connect_pwd		varchar2(10);
	a_lin			varchar2(80);
	lv_lineno		number;

	function wri(x_lin in varchar2, x_str in varchar2,
		x_force in number) return varchar2 is
	begin
		if length(x_lin) + length(x_str) > 80
		then
			lv_lineno := lv_lineno + 1;
			insert into l_temp values (lv_lineno, x_lin);
			if x_force = 0
			then
				return '    '||x_str;
			else
				lv_lineno := lv_lineno + 1;
				if substr(x_lin,1,2) = '  '
				then
					insert into l_temp values (
						lv_lineno, x_str);
				else
					insert into l_temp values (
						lv_lineno, '    '||x_str);
				end if;
				return '';
			end if;
		else
			if x_force = 0
			then
				return x_lin||x_str;
			else
				lv_lineno := lv_lineno + 1;
				insert into l_temp values (
					lv_lineno, x_lin||x_str);
				return '';
			end if;
		end if;
	end wri;

begin
	lv_lineno := 0;
	a_lin := '';
	prev_owner := '@';
	open link_cursor;
	loop
		fetch link_cursor into
			lv_owner,
			lv_db_link,
			lv_username,
			lv_password,
			lv_host;
		exit when link_cursor%NOTFOUND;
		/* */
		/* Since we can not create a database link in another user's */
		/* schema, we need to build in the connect strings to switch */
		/* to the user's schema who owns the private database link */
		/* */
		if lv_owner != prev_owner then
			/* Reset password if we changed it */
			if prev_owner != '@' then
				a_lin := wri(a_lin, 'connect /', 1);
				if user_password != '<password>' then
					a_lin := wri(a_lin, 'alter user '||
						lower(prev_owner)||
						' identified by values '||&q||
						user_password||&q||';', 1);
				else
					a_lin := wri(a_lin, 'alter user '||
						lower(prev_owner)||
						' identified by <password>;',
						1);
				end if;
			end if;
			if lv_owner = 'PUBLIC' then
				open pwd_cursor('SYSTEM');
				fetch pwd_cursor into user_password;
				if pwd_cursor%NOTFOUND then
					user_password := '<password>';
					dbms_output.put_line(
						'*****> Warning:  Username '||
						'SYSTEM'||
						' not found in DBA_USERS!!');
				end if;
			else
				open pwd_cursor(lv_owner);
				fetch pwd_cursor into user_password;
				if pwd_cursor%NOTFOUND then
					user_password := '<password>';
					dbms_output.put_line(
						'*****> Warning:  Username '||
						lv_owner||
						' not found in DBA_USERS!!');
				end if;
			end if;
			close pwd_cursor;
			a_lin := wri(a_lin, ' ', 1);
			a_lin := wri(a_lin, 'rem ----------------------------',
				1);
			a_lin := wri(a_lin, ' ', 1);
			if lv_owner = 'PUBLIC' then
				alter_user := 'system';
			else
				alter_user := lower(lv_owner);
			end if;
			if user_password = '<password>' then
				connect_pwd := user_password;
				a_lin := wri(a_lin, 'alter user '||alter_user||
					' identified by <password>;', 1);
			else
				connect_pwd := 'xyzzy';
				a_lin := wri(a_lin, 'alter user '||alter_user||
					' identified by xyzzy;', 1);
			end if;
			a_lin := wri(a_lin, 'connect '||alter_user||'/'||
				connect_pwd, 1);
			if lv_owner = 'PUBLIC' then
				prev_owner := 'system';
			else
				prev_owner := lv_owner;
			end if;
		end if;
		if lv_owner = 'PUBLIC' then
			a_lin := wri(a_lin, 'CREATE PUBLIC DATABASE LINK '||
				lv_db_link, 1);
		else
			a_lin := wri(a_lin, 'CREATE DATABASE LINK '||
				lv_db_link, 1);
		end if;
		a_lin := wri(a_lin, '    ', 0);
		if lv_username != ' ' then
			a_lin := wri(a_lin, ' connect to '||lv_username
				||' identified by '||lv_password, 0);
		end if;
		if lv_host != ' ' then
			a_lin := wri(a_lin, ' using '||&q||lv_host||&q, 0);
		end if;
		a_lin := wri(a_lin, ';', 1);
	end loop;
	close link_cursor;
	/* Reset password if we changed it */
	if prev_owner != '@' then
		a_lin := wri(a_lin, 'connect /', 1);
		if user_password != '<password>' then
			a_lin := wri(a_lin, 'alter user '||lower(prev_owner)||
				' identified by values '||&q||user_password||
				&q||';', 1);
		else
			a_lin := wri(a_lin, 'alter user '||lower(prev_owner)||
				' identified by <password>;', 1);
		end if;
	end if;
	a_lin := wri(a_lin, 'exit', 1);
end;
/

set termout off
set heading off
spool cr_link.sql
select 'rem cr_link.sql' from dual;
select 'rem' from dual;
select 'rem ***** All database links for database ' || name from v$database;
select 'rem' from dual;
select 'set feedback off' from dual;
select text from l_temp order by lineno;
spool off
drop table l_temp;
set termout on
select 'Created cr_link.sql...' from dual;
set termout off
exit
