rem $DBA/chktable.sql
rem
rem Checks a specific table's CHAR and VARCHAR2 columns for abnormal characters
rem (i.e.  quotes, apostrophes, or control characters).
rem
rem Prompts for the table owner and the table name to be checked.
rem
rem Requires Oracle V7.2.2.3 (since dynamic SQL in PL/SQL version 2.2.2.3 is
rem used)
rem
rem A maximum of 10 CHAR or 10 VARCHAR2 columns can be checked in a single
rem table.  (Use chktabl2.sql if more than 10 columns).
rem
rem Created 06/24/97 by Brian Lomasky
rem
set echo off
set feedback off
set pagesize 0
set verify off
set serveroutput on size 100000
set termout on
declare
	cursor col_cursor is select
		owner,
		table_name,
		column_name,
		data_type,
		data_length
		from sys.dba_tab_columns
		where owner = upper('&&table_owner') and
		table_name = upper('&&table_name')
		order by column_id;

	lv_owner		sys.dba_tab_columns.owner%TYPE;
	lv_table_name		sys.dba_tab_columns.table_name%TYPE;
	lv_column_name		sys.dba_tab_columns.column_name%TYPE;
	lv_data_type		sys.dba_tab_columns.data_type%TYPE;
	lv_data_length		sys.dba_tab_columns.data_length%TYPE;
	the_owner		sys.dba_tab_columns.owner%TYPE;
	the_table_name		sys.dba_tab_columns.table_name%TYPE;
	stmnt			varchar2(2000);
	abnormal		number;
	n			number;
	i			number;
	a			number;
	rows_processed		number;
	rowno			number;
	cursor1			integer;
	out_char		char(2000);
	out_char1		char(2000);
	out_char2		char(2000);
	out_char3		char(2000);
	out_char4		char(2000);
	out_char5		char(2000);
	out_char6		char(2000);
	out_char7		char(2000);
	out_char8		char(2000);
	out_char9		char(2000);
	out_char10		char(2000);
	out_varchar2		varchar2(2000);
	out_varchar21		varchar2(2000);
	out_varchar22		varchar2(2000);
	out_varchar23		varchar2(2000);
	out_varchar24		varchar2(2000);
	out_varchar25		varchar2(2000);
	out_varchar26		varchar2(2000);
	out_varchar27		varchar2(2000);
	out_varchar28		varchar2(2000);
	out_varchar29		varchar2(2000);
	out_varchar210		varchar2(2000);
	arr_ndx			number;
	arr_elem		number;
	no_char_cols		exception;
	too_many		exception;

	type colntyp is table of lv_column_name%TYPE index by binary_integer;
	type dttyp is table of lv_data_type%TYPE index by binary_integer;
	type lentyp is table of lv_data_length%TYPE index by binary_integer;

	coln			colntyp;
	dt			dttyp;
	dlen			lentyp;

begin
	arr_ndx := 0;
	open col_cursor;
	loop
		fetch col_cursor into
			lv_owner,
			lv_table_name,
			lv_column_name,
			lv_data_type,
			lv_data_length;
		exit when col_cursor%notfound;
		the_owner := lv_owner;
		the_table_name := lv_table_name;
		if lv_data_type = 'CHAR' or lv_data_type = 'VARCHAR2' then
			arr_ndx := arr_ndx + 1;
			coln(arr_ndx) := lv_column_name;
			dt(arr_ndx) := lv_data_type;
			dlen(arr_ndx) := lv_data_length;
		end if;
	end loop;
	close col_cursor;
	if arr_ndx = 0 then
		dbms_output.put_line('No CHAR or VARCHAR2 columns found in ' ||
			upper('&&table_owner') || '.' ||
			upper('&&table_name'));
		raise no_char_cols;
	end if;

	cursor1 := dbms_sql.open_cursor;

	arr_elem := 0;
	while arr_elem < arr_ndx
	loop
		arr_elem := arr_elem + 1;
		dbms_output.put_line('Processing column name: ' ||
			coln(arr_elem) || '...');
		if arr_elem = 1 then
			stmnt := 'select ' || coln(arr_elem);
		else
			stmnt := stmnt || ', ' || coln(arr_elem);
		end if;
	end loop;
	stmnt := stmnt || ' from ' || the_owner || '.' || the_table_name;
	dbms_sql.parse(cursor1, stmnt, dbms_sql.native);

	arr_elem := 0;
	while arr_elem < arr_ndx
	loop
		arr_elem := arr_elem + 1;
		if dt(arr_elem) = 'CHAR' then
			if arr_elem = 1 then
				dbms_sql.define_column(cursor1, arr_elem,
					out_char1, dlen(arr_elem));
			elsif arr_elem = 2 then
				dbms_sql.define_column(cursor1, arr_elem,
					out_char2, dlen(arr_elem));
			elsif arr_elem = 3 then
				dbms_sql.define_column(cursor1, arr_elem,
					out_char3, dlen(arr_elem));
			elsif arr_elem = 4 then
				dbms_sql.define_column(cursor1, arr_elem,
					out_char4, dlen(arr_elem));
			elsif arr_elem = 5 then
				dbms_sql.define_column(cursor1, arr_elem,
					out_char5, dlen(arr_elem));
			elsif arr_elem = 6 then
				dbms_sql.define_column(cursor1, arr_elem,
					out_char6, dlen(arr_elem));
			elsif arr_elem = 7 then
				dbms_sql.define_column(cursor1, arr_elem,
					out_char7, dlen(arr_elem));
			elsif arr_elem = 8 then
				dbms_sql.define_column(cursor1, arr_elem,
					out_char8, dlen(arr_elem));
			elsif arr_elem = 9 then
				dbms_sql.define_column(cursor1, arr_elem,
					out_char9, dlen(arr_elem));
			elsif arr_elem = 10 then
				dbms_sql.define_column(cursor1, arr_elem,
					out_char10, dlen(arr_elem));
			elsif arr_elem = 11 then
				raise too_many;
			end if;
		else
			if arr_elem = 1 then
				dbms_sql.define_column(cursor1, arr_elem,
					out_varchar21, dlen(arr_elem));
			elsif arr_elem = 2 then
				dbms_sql.define_column(cursor1, arr_elem,
					out_varchar22, dlen(arr_elem));
			elsif arr_elem = 3 then
				dbms_sql.define_column(cursor1, arr_elem,
					out_varchar23, dlen(arr_elem));
			elsif arr_elem = 4 then
				dbms_sql.define_column(cursor1, arr_elem,
					out_varchar24, dlen(arr_elem));
			elsif arr_elem = 5 then
				dbms_sql.define_column(cursor1, arr_elem,
					out_varchar25, dlen(arr_elem));
			elsif arr_elem = 6 then
				dbms_sql.define_column(cursor1, arr_elem,
					out_varchar26, dlen(arr_elem));
			elsif arr_elem = 7 then
				dbms_sql.define_column(cursor1, arr_elem,
					out_varchar27, dlen(arr_elem));
			elsif arr_elem = 8 then
				dbms_sql.define_column(cursor1, arr_elem,
					out_varchar28, dlen(arr_elem));
			elsif arr_elem = 9 then
				dbms_sql.define_column(cursor1, arr_elem,
					out_varchar29, dlen(arr_elem));
			elsif arr_elem = 10 then
				dbms_sql.define_column(cursor1, arr_elem,
					out_varchar210, dlen(arr_elem));
			elsif arr_elem = 11 then
				raise too_many;
			end if;
		end if;
	end loop;

	rows_processed := dbms_sql.execute(cursor1);
	abnormal := 0;
	rowno := 0;
	loop
		if dbms_sql.fetch_rows(cursor1) = 0 then
			exit;
		end if;
		rowno := rowno + 1;
		arr_elem := 0;
		while arr_elem < arr_ndx
		loop
			arr_elem := arr_elem + 1;
			if dt(arr_elem) = 'CHAR' then
				dbms_sql.column_value(cursor1, arr_elem,
					out_char);
				out_varchar2 := substr(out_char, 1,
					dlen(arr_elem));
			else
				dbms_sql.column_value(cursor1, arr_elem,
					out_varchar2);
			end if;
			/* ---------- Check for quote ---------- */
			n := instr(out_varchar2, chr(34));
			if n <> 0 then
				dbms_output.put_line('.  Row ' ||
					to_char(rowno) || ' of column ' ||
					coln(arr_elem) ||
					' has an embedded quote' ||
					' at position ' || to_char(n));
				abnormal := 1;
				exit;
			end if;
			/* ---------- Check for apostrophe ---------- */
			n := instr(out_varchar2, chr(39));
			if n <> 0 then
				dbms_output.put_line('.  Row ' ||
					to_char(rowno) || ' of column ' ||
					coln(arr_elem) ||
					' has an embedded apostrophe' ||
					' at position ' || to_char(n));
				abnormal := 1;
				exit;
			end if;
			/* ---------- Check for any control chars ---------- */
			i := 0;
			while i < length(out_varchar2)
			loop
				i := i + 1;
				a := ascii(substr(out_varchar2,i,1));
				if a < 32 then
					dbms_output.put_line('.  Row ' ||
						to_char(rowno) ||
						' of column ' ||
						coln(arr_elem) ||
						' has an ASCII character of' ||
						' <' || to_char(a) ||
						'> at position ' || to_char(n));
					abnormal := 1;
					exit;
				end if;
			end loop;
			if abnormal <> 0 then
				exit;
			end if;
		end loop;
		if abnormal <> 0 then
			exit;
		end if;
	end loop;
	if abnormal = 0 then
		dbms_output.put_line(
			'.  (No abnormal characters found in ' ||
			to_char(rowno) || ' rows...)');
	end if;

	dbms_sql.close_cursor(cursor1);

	commit;
exception
	when no_char_cols then
		commit;
	when too_many then
		commit;
		if dbms_sql.is_open(cursor1) then
			dbms_sql.close_cursor(cursor1);
		end if;
                raise_application_error(-20000,
                        'Error - Too many CHAR or VARCHAR2 columns in the' ||
			' table - Aborting...');
        when others then
                rollback;
		if dbms_sql.is_open(cursor1) then
			dbms_sql.close_cursor(cursor1);
		end if;
                raise_application_error(-20000,
                        'Unexpected error on column ' || coln(arr_elem) ||
			chr(10) || SQLERRM || chr(10) || 'Aborting...');
end;
/
exit
