/* GRANT_TO.SQL */

/* Lists all direct grants made to a specific user or role, segregated by */
/* the nature of the grant.  This report is a complete listing of all     */
/* initial permissions along all access paths for a user.                 */
/* Used to research the potential impact of revoking any direct grant     */
/* issued to a user or role.                                              */

/* Input:  A user or role name (including PUBLIC) */

/* Output Column Legend: */
/*   LEVEL indicates the type of grant issued. */
/*   PERMISSION contains the privilege granted for all levels except for */
/*              Role.  The permission listed in a role level grant is the */
/*              name of the role that was granted. */

set pagesize 66
set verify off
set feedback off
column lvl format A8 Heading "Level"
column privilege format A15
column grantable format A4 heading "Gtbl"
column owner format A9
column table_name format A26 heading "Table Name"
column column_name format A10 heading "Col Name"
ttitle left 'GRANT_TO - Direct Grants made to '&1 skip 2
spool grant_to.lst
set termout off
SELECT 'Column' lvl, c.privilege, c.grantable, c.owner, c.table_name,
       c.column_name
FROM dba_col_privs c
WHERE c.grantee = UPPER('&1') AND c.owner != 'SYS'
UNION
SELECT 'Role' Gr_Type, r.granted_role obj, r.admin_option a, NULL, NULL, NULL
FROM dba_role_privs r
WHERE r.grantee = UPPER('&1')
UNION
SELECT 'Sys Priv', s.privilege, s.admin_option, NULL, NULL, NULL
FROM dba_sys_privs s
WHERE s.grantee = UPPER('&1')
UNION
SELECT 'Table', t.privilege, t.grantable, t.owner, t.table_name, NULL
FROM dba_tab_privs t
WHERE t.grantee = UPPER('&1') AND t.privilege != 'EXECUTE' AND
      t.owner != 'SYS'
UNION
SELECT 'Program', e.privilege, e.grantable, e.owner, e.table_name, NULL
FROM dba_tab_privs e
WHERE e.grantee = UPPER('&1') AND e.privilege = 'EXECUTE' AND
      e.owner != 'SYS'
ORDER BY 1, 2, 3, 4, 5, 6;
spool off
set termout on
/* prompt Created grant_to.lst for your viewing pleasure... */
exit
