rem DBA:PRIVLIST.SQL

rem List the security objects along all access paths that exist for the
rem specified user (role) by drilling through all levels of indirection.
rem Used to determine a complete list of security objects for a user.
rem The source of each individual privilege is listed so that the proper
rem REVOKE statement can be issued to deny the listed privileges.

rem Input:  &&1 = A user or role name (including PUBLIC)

rem Output Column Legend:
rem   OWNER is the fully qualified name of the table, view, or program
rem         to which the user has been granted privileges.
rem   PRIVILEGE is the object privilege that has been granted.
rem   Direct to User = Privilege was acquired by a grant directly to the user
rem   ADMIN is Y is the user has been granted that privilege WITH GRANT OPTION

set verify off
set pagesize 9999
set feedback on
column owner format A13
column table_name format A28
column privilege format A12
column granted_role format A15
column adm format A5 heading "ADMIN"
ttitle left 'PRIVLIST - List Privileges for &&1 in '_NAME'        '_DATE skip 2
spool privlist.lis
set termout off
SELECT DISTINCT t.owner, t.table_name, t.privilege, r.granted_role,
	 decode(r.admin_option, 'Y', 'Y', ' ') adm
FROM dba_tab_privs t, dba_role_privs r
WHERE t.grantee IN
 (SELECT granted_role FROM dba_role_privs WHERE grantee = UPPER('&&1')
  UNION
  SELECT granted_role FROM dba_role_privs WHERE grantee = 'PUBLIC'
  UNION
  SELECT granted_role FROM dba_role_privs WHERE grantee IN
   (SELECT granted_role FROM dba_role_privs WHERE grantee = UPPER('&&1')))
  AND t.grantee = r.granted_role
  AND t.owner NOT IN ('SYSTEM', 'SYS')
UNION
SELECT owner, table_name, privilege, 'Direct to User',
	decode(grantable, 'YES', 'Y', ' ')
FROM dba_tab_privs
WHERE grantee = UPPER('&&1') AND owner NOT IN ('SYSTEM', 'SYS')
UNION
SELECT owner, table_name, privilege, 'Public',
	decode(grantable, 'YES', 'Y', ' ')
FROM dba_tab_privs
WHERE grantee = 'PUBLIC' AND owner NOT IN ('SYSTEM', 'SYS')
UNION
SELECT owner, table_name, privilege, 'Column Security',
	decode(grantable, 'YES', 'Y', ' ')
FROM dba_col_privs
WHERE grantee = UPPER('&&1')
UNION
SELECT DISTINCT c.owner, c.table_name, '(Column)', r.granted_role,
	 decode(r.admin_option, 'Y', 'Y', ' ')
FROM dba_col_privs c, dba_role_privs r
WHERE c.grantee IN
 (SELECT granted_role FROM dba_role_privs WHERE grantee = UPPER('&&1')
  UNION
  SELECT granted_role FROM dba_role_privs WHERE grantee = 'PUBLIC'
  UNION
  SELECT granted_role FROM dba_role_privs WHERE grantee IN
   (SELECT granted_role FROM dba_role_privs WHERE grantee = UPPER('&&1')))
  AND c.grantee = r.granted_role
  AND c.owner NOT IN ('SYSTEM', 'SYS')
UNION
SELECT DISTINCT owner, table_name, '(Column)', '(Public)',
	decode(grantable, 'YES', 'Y', ' ')
FROM dba_col_privs
WHERE grantee = 'PUBLIC' ORDER BY 2, 1, 3;
spool off
set termout on
prompt Created privlist.lis for your viewing pleasure...
exit
