/* ROLELIST.SQL */

/* Lists all roles in the database and to whom (user or role) they were     */
/* granted.  This report is used to walk backwards along all access paths   */
/* in the database to their start points (user names).  Used when           */
/* researching the levels of security indirection in a database.            */

/* Input:  None */

/* Output Column Legend: */
/*   ROLE contains the role names in the database */
/*   PWD indicates whether the role is password protected */
/*   GRANTED_TO contains the user or role to which the role was granted */
/*   ADM is the Admin Option value from the grant statement */
/*   DFLT indicates whether the role is a default role */

set pagesize 66
set verify off
set feedback off
column role format A27
column Pwd format A9 Heading "Password|Protected"
column grantee format A15
column adm format A6 Heading "Admin|Option"
column dflt format A7 Heading "Default|Role?"
ttitle left 'ROLELIST - List of all roles' skip 2
spool rolelist.lst
set termout off
SELECT r.role, r.password_required Pwd, p.grantee, p.admin_option adm,
       p.default_role dflt
FROM dba_roles r, dba_role_privs p
WHERE r.role = p.granted_role(+)
ORDER BY 1, 3;
spool off
set termout on
/* prompt Created rolelist.lst for your viewing pleasure... */
exit
