rem $DBA/tablecon.sql
rem
rem Prints a report of the referential integrity constraints for a specific
rem table (&1)
rem
rem Last Change 10/28/96 by Brian Lomasky
rem
set feedback off
set heading off 
set echo off
set pagesize 0
set newpage 0
set linesize 80
set recsep off
set verify off 
set termout on

rem Create temp table for output report
create table tablecon_temp (
	linen		number,
	lin		varchar2(80));

set termout off
declare
	t_owner			sys.dba_constraints.owner%TYPE;
	t_constraint_name	sys.dba_constraints.constraint_name%TYPE;
	t_constraint_type	sys.dba_constraints.constraint_type%TYPE;
	t_table_name		sys.dba_constraints.table_name%TYPE;
	t_search_condition	sys.dba_constraints.search_condition%TYPE;
	t_r_owner		sys.dba_constraints.r_owner%TYPE;
	t_r_table		sys.dba_constraints.table_name%TYPE;
	t_r_constraint_name	sys.dba_constraints.r_constraint_name%TYPE;
	t_delete_rule		sys.dba_constraints.delete_rule%TYPE;
	t_status		sys.dba_constraints.status%TYPE;
	lineno			number;
	nn_pos			number;
	columns_reqd		varchar2(1);
	text_length		number;
	startp			number;
	xchar			number;
	break_pos		number;
	lf_pos			number;
	semi_pos		number;
	lf_break		number;
	backwords		number;
	new_line		number;
	offset			number;
	out_start		number;
	out_len			number;
	l			number;
	out_line		varchar2(2000);
	bef_chars		varchar2(2000);
	a_lin			varchar2(2000);
	my_lin			varchar2(2000);
	search_for_break	boolean;
	start_break_search	number;

	cursor c1 is
		select	dc1.owner,
			dc1.table_name,
			dc1.constraint_name,
			dc1.constraint_type,
			dc1.search_condition,
			dc2.table_name r_table,
			dc1.r_owner,
			dc1.r_constraint_name,
			dc1.delete_rule,
			dc1.status
		from	dba_constraints dc1, dba_constraints dc2
		where	dc1.r_constraint_name = dc2.constraint_name (+) and
			dc1.table_name = '&&1' and
			dc1.constraint_name not like 'SYS%' and
			dc1.constraint_type <> 'P' and
			dc1.constraint_type <> 'U'
		order by
			decode(dc1.constraint_type, 'R', 1, 2),
			dc1.owner, dc1.table_name, dc1.constraint_name;

	function wri(x_lin in varchar2, x_str in varchar2, x_force in number)
		return varchar2 is
	begin
		if length(x_lin) + length(x_str) > 80
		then
			lineno := lineno + 1;
			insert into tablecon_temp values (lineno, x_lin);
			if x_force = 0
			then
				return '    '||x_str;
			else
				lineno := lineno + 1;
				if substr(x_lin,1,2) = '  '
				then
					insert into tablecon_temp values (
						lineno, x_str);
				else
					insert into tablecon_temp values (
						lineno, '    '||x_str);
				end if;
				return '';
			end if;
		else
			if x_force = 0
			then
				return x_lin||x_str;
			else
				lineno := lineno + 1;
				insert into tablecon_temp values (
					lineno, x_lin||x_str);
				return '';
			end if;
		end if;
	end wri;

	function brkline(x_lin in varchar2, x_str in varchar2,
		x_force in number) return varchar2 is
	begin
		my_lin := x_lin;
		text_length := nvl(length(x_str), 0);
		-- Break long text field into smaller groups to print
		startp := 1;
		while startp <= text_length
		loop
			-- Assume not backing up to first space
			backwords := 0;
			offset := 0;
			-- Assume forcing a new line
			new_line := 1;
			-- Break line at next space/tab, semi-colon, linefeed,
			-- 80 chars, or end of string
			search_for_break := TRUE;
			start_break_search := startp;
			while search_for_break
			loop
				search_for_break := FALSE;
				break_pos := instr(x_str, ' '||chr(9),
					start_break_search);
				-- Search for next space/tab if no characters
				-- come before it
				if break_pos > 0
				then
					bef_chars := ltrim(substr(x_str,
						start_break_search,
						break_pos - start_break_search +
						1));
					if nvl(bef_chars, '@@xyzzy') = '@@xyzzy'
					then
						break_pos := 0;
						if start_break_search + 2 <
							text_length
						then
							search_for_break :=
							    TRUE;
							start_break_search :=
							    start_break_search
							    + 1;
						end if;
					end if;
				end if;
			end loop;
			lf_pos := instr(x_str, chr(10), startp);
			lf_break := 0;
			-- See if a linefeed break comes before space/tab break
			if (lf_pos < break_pos or break_pos = 0) and lf_pos > 0
			then
				-- Use linefeed as break position search
				break_pos := lf_pos;
				lf_break := 1;
			end if;
			semi_pos := instr(x_str, ';', startp);
			if break_pos + lf_pos = 0 or (break_pos > semi_pos and
				semi_pos > 0)
			then
				if semi_pos = 0
				then
					-- Break line at 80 chars
					break_pos := startp + 80;
					if break_pos > text_length
					then
						break_pos := text_length + 1;
					end if;
					-- Back up to first space
					backwords := 1;
					-- Do not force new line
					new_line := 0;
				else
					-- Assume line breaking at semi-colon
					break_pos := semi_pos + 1;
				end if;
			else
				if lf_break = 0 then
					-- Assume line breaking at space/tab
					break_pos := break_pos + 1;
					-- Skip space part of the space/tab
					offset := 1;
				else
					-- Assume line breaking before linefeed
					-- Skip linefeed
					offset := 1;
				end if;
			end if;
			if break_pos - startp > 80
			then
				break_pos := startp + 79;
				if break_pos > text_length
				then
					break_pos := text_length + 1;
				end if;
				-- Back up to first space
				backwords := 1;
			end if;
			while backwords = 1
			loop
				if break_pos > text_length
				then
					backwords := 0;
					exit;
				end if;
				if break_pos <= startp
				then
					break_pos := startp + 79;
					if break_pos > text_length
					then
						break_pos := text_length + 1;
					end if;
					backwords := 0;
					exit;
				end if;
				if substr(x_str, break_pos, 1) = ' '
				then
					backwords := 0;
					exit;
				end if;
				break_pos := break_pos - 1;
			end loop;
			xchar := break_pos - startp;
			if xchar = 0 then
				if offset = 0 then
					return my_lin;
				end if;
			else
				-- Replace all tabs with 8 spaces in output line
				out_line := replace(substr(x_str, startp,
					xchar), chr(9), '        ');
				-- In case replacing the tabs with spaces caused
				-- the output line to exceed 80 chars, write
				-- 80-col pieces of the translated output line
				out_start := 1;
				l := length(out_line);
				if nvl(l, -1) = -1 then
					return my_lin;
				end if;
				while out_start <= l
				loop
					-- Write output line
					if l >= out_start + 79 then
						out_len := 80;
					else
						out_len := l - out_start + 1;
					end if;
					my_lin := wri(my_lin,
						substr(out_line, out_start,
						out_len), new_line);
					out_start := out_start + out_len;
				end loop;
			end if;
			startp := startp + xchar + offset;
		end loop;
		return my_lin;
	end brkline;
begin
	lineno := 0;
	a_lin := '';
	open c1;
	loop
		fetch c1 into t_owner, t_table_name, t_constraint_name,
			t_constraint_type, t_search_condition, t_r_table,
			t_r_owner, t_r_constraint_name, t_delete_rule, t_status;
		exit when c1%notfound;
		columns_reqd := 'n';
		if t_constraint_type = 'C'
		then
			nn_pos := instr(t_search_condition, ' IS NOT NULL');
			if nn_pos = 0 then
				a_lin := wri(a_lin, '     Check: ', 0);
				a_lin := brkline(a_lin, t_search_condition, 0);
			else
				a_lin := wri(a_lin, '     Not Null on ', 0);
				a_lin := wri(a_lin, substr(t_search_condition,
					1, nn_pos - 1), 0);
			end if;
			if t_status = 'DISABLED' then
				a_lin := wri(a_lin, ' DISABLE', 0);
			end if;
			a_lin := wri(a_lin, ' ', 1);
		end if;
		if t_constraint_type = 'R'
		then
			a_lin := wri(a_lin, '     Foreign key on ', 0);
			columns_reqd := 'Y';
		end if;
		if columns_reqd = 'Y'
		then
			declare
				c_owner
				    sys.dba_cons_columns.owner%TYPE;
				c_constraint_name
				    sys.dba_cons_columns.constraint_name%TYPE;
				c_table_name
				    sys.dba_cons_columns.table_name%TYPE;
				c_column_name
				    sys.dba_cons_columns.column_name%TYPE;
				c_position
				    sys.dba_cons_columns.position%TYPE;
				cursor c2 is
					select owner, constraint_name,
						table_name, column_name,
						position
					from dba_cons_columns
					where owner = t_owner and
					      constraint_name =
					      t_constraint_name and
					      table_name = t_table_name
	         			order by position;
			begin
				open c2;
				loop
					fetch c2 into c_owner,
						c_constraint_name, c_table_name,
						c_column_name, c_position;
					exit when c2%notfound;
					if c_position > 1 then
						a_lin := wri(a_lin,
							'                    ',
							0);
					end if;
					a_lin := wri(a_lin, c_column_name, 1);
				end loop;
				close c2;
			end;
			if t_constraint_type = 'R'
			then
				declare
				c_owner	sys.dba_cons_columns.owner%TYPE;
				c_table_name
					sys.dba_cons_columns.table_name%TYPE;
				cursor c3 is
					select owner, table_name
					from dba_cons_columns
					where owner = t_r_owner and
						constraint_name =
						t_r_constraint_name
	         			order by position;
				begin
					open c3;
					loop
						fetch c3 into c_owner,
							c_table_name;
						exit when c3%notfound;
					end loop;
					close c3;
					a_lin := wri(a_lin,
						'         references ', 0);
					a_lin := wri(a_lin, c_owner||'.'||
						c_table_name, 1);
				end;
			end if;
		end if;
		if t_constraint_type = 'R'
		then
			declare
				c_owner
				    sys.dba_cons_columns.owner%TYPE;
				c_constraint_name
				    sys.dba_cons_columns.constraint_name%TYPE;
				c_table_name
				    sys.dba_cons_columns.table_name%TYPE;
				c_column_name
				    sys.dba_cons_columns.column_name%TYPE;
				c_position
				    sys.dba_cons_columns.position%TYPE;
				cursor c4 is
					select owner, constraint_name,
						table_name, column_name,
						position
					from dba_cons_columns
					where owner = t_r_owner and
					      constraint_name =
					      t_r_constraint_name and
					      table_name = t_r_table
	         			order by position;
			begin
				open c4;
				loop
					fetch c4 into c_owner,
						c_constraint_name, c_table_name,
						c_column_name, c_position;
					exit when c4%notfound;
					a_lin := wri(a_lin,
						'                      ', 0);
					a_lin := wri(a_lin, c_column_name, 1);
				end loop;
				close c4;
			end;
			if t_delete_rule = 'CASCADE' then
				a_lin := wri(a_lin,
					'                      ' ||
					'on delete cascade', 1);
			end if;
			if t_status = 'DISABLED' then
				a_lin := wri(a_lin,
					'                      ' ||
					'DISABLE', 1);
			end if;
		end if;
	end loop;
	close c1;
	commit;
end;
/
select rtrim(lin) from tablecon_temp order by linen;
drop table tablecon_temp;
exit
