rem $DBA/privs.sql
rem
rem Prints a privilege report for the specified user, listing the granted roles,
rem and the individual object privileges granted to the user or PUBLIC.  Smaller
rem report than privlist.sql, since the granted roles do not have their
rem privileges broken out, and the DELETE/INSERT/SELECT/UPDATE appear once per
rem table.
rem
rem Calls $DBA/privsdt.sql and $DBA/privsdc.sql
rem
rem Input:  &&1 = A user or role name (including PUBLIC)
rem
rem Last Change 09/17/97 by Brian Lomasky
rem
set echo off
set heading off
set feedback off
set termout off
set verify off
column name new_value _name
select name from v$database;
set heading on
set pagesize 9999
column owner format A13
column table_name format A28
column privilege format A12
column granted_role format A25
column adm format A5 heading "ADMIN"
ttitle 'Roles granted to &&1 in '_NAME skip 2
spool privs1.lis
set termout off
SELECT granted_role
	FROM dba_role_privs
	WHERE grantee = UPPER('&1')
	ORDER BY 1;
ttitle 'Roles granted to PUBLIC in '_NAME skip 2
SELECT granted_role
	FROM dba_role_privs
	WHERE grantee = 'PUBLIC' and UPPER('&1') <> 'PUBLIC'
	ORDER BY 1;
ttitle 'Roles granted to roles which are granted to &&1' skip 2
SELECT granted_role
	FROM dba_role_privs
	WHERE grantee in (SELECT granted_role FROM dba_role_privs
			  WHERE grantee = UPPER('&1'))
	ORDER BY 1;
spool off
@$DBA/privsdt &&1
@$DBA/privsdc &&1
set termout on
prompt Created privs.lis for your viewing pleasure...
exit
