REM ******************************************************
REM File:  Q100_110.DDL
REM
REM Makes QDBA schema modifications for migrating from
REM Q v1.0 to Q v1.1
REM ******************************************************
REM ******************************************************
REM Revision History:
REM
REM 11/24/97: added PRIMARY KEY constraint to id col [JMB]
REM           of q$bgp_config
REM 06/19/97: added obj# to q$seg                    [JMB]
REM 06/19/97: watchcount and last_watch_date moved   [JMB]
REM           into q$seg table
REM 06/19/97: updated watchcount to 1 for new use    [JMB]
REM 06/19/97: added last_watch_date to q$segwatch    [JMB]
REM 06/19/97: defaulting to TEMP_SEGS for segwatch   [JMB]
REM 06/17/97: added file# and block# to q$seg and    [JMB]
REM           q$segwatch tables
REM 06/16/97: modified q$seg_PK column order for     [JMB]
REM           performance of tempseg processing
REM 06/14/97: re-CREATE q$bgp_config to redefine     [JMB]
REM           and name all CHECK constraints
REM 06/14/97: original creation                      [JMB]
REM ******************************************************

CREATE TABLE &&qdba..q$bgp_config_temp
   TABLESPACE &&qtablespace
STORAGE (INITIAL 	1 
	      NEXT 		1
         MINEXTENTS  1
 	      PCTINCREASE 	0)
AS 
   SELECT * FROM &&qdba..q$bgp_config;

UPDATE &&qdba..q$bgp_config_temp 
   SET segwatch=DECODE(segwatch,'SEGWATCH','TEMP_SEGS',segwatch);

DROP TABLE &&qdba..q$bgp_config;

CREATE TABLE &&qdba..q$bgp_config
   (id               VARCHAR2(12)   NOT NULL PRIMARY KEY 
   ,collect          VARCHAR2(12)   NOT NULL
            CONSTRAINT ck_collect 
            CHECK ( collect IN ('COLLECT','NO_COLLECT') )
   ,bench            VARCHAR2(12)   NOT NULL
            CONSTRAINT ck_bench
            CHECK ( bench IN ('BENCH','NO_BENCH') )
   ,alarm            VARCHAR2(12)   NOT NULL
            CONSTRAINT ck_alarm
            CHECK ( alarm IN ('ALARM','NO_ALARM') )
   ,segwatch         VARCHAR2(12)   NOT NULL
            CONSTRAINT ck_segwatch
            CHECK ( segwatch IN ('TEMP_SEGS','ALL_SEGS','NO_SEGWATCH') )
   ,keep_pkgs        VARCHAR2(12)   NOT NULL
            CONSTRAINT ck_keep_pkgs
            CHECK ( keep_pkgs IN ('KEEP','NO_KEEP') )
   ,collect_interval INTEGER        NOT NULL
            CONSTRAINT ck_collect_interval
            CHECK ( collect_interval > 0 )
   ,bench_interval   INTEGER        NOT NULL
            CONSTRAINT ck_bench_interval
            CHECK ( bench_interval > 0 )
   ,alarm_interval   INTEGER        NOT NULL
            CONSTRAINT ck_alarm_interval
            CHECK ( alarm_interval > 0 )
   ,detail_mode      VARCHAR2(12)   NOT NULL
            CONSTRAINT ck_detail_mode
            CHECK ( detail_mode IN ('DAILY','MANUAL','CONTINUOUS') )
   ,detail_purge     VARCHAR2(12)   NOT NULL
            CONSTRAINT ck_detail_purge
            CHECK ( detail_purge IN ('PURGE','NO_PURGE') )
   ,detail_start_date  DATE         NULL
   ,detail_end_date    DATE         NULL
   )
TABLESPACE &&qtablespace
PCTFREE	5
STORAGE (INITIAL 	1 
	      NEXT 		1
         MINEXTENTS  1
 	      PCTINCREASE 	0);

CREATE OR REPLACE TRIGGER &&qdba..q$bgp_config_UDbr 
BEFORE UPDATE OR DELETE
ON &&qdba..q$bgp_config
FOR EACH ROW
BEGIN
   IF UPDATING AND :NEW.id <> :OLD.id 
   THEN 
      raise_application_error(-20001, 'Cannot update ID');
   END IF;
   IF DELETING AND :OLD.id IN ('DEFAULT','CURRENT')
   THEN
      raise_application_error(-20001, 'Cannot DELETE id: '||:OLD.id);
   END IF;   
END;
/

INSERT INTO &&qdba..q$bgp_config
SELECT * FROM &&qdba..q$bgp_config_temp;

DROP TABLE &&qdba..q$bgp_config_temp;

REM *******************************************
REM re-arrange columns of q$seg_PK to improve
REM performance of Q$SPACE.PROCESS_TEMPSEGS
REM *******************************************

ALTER TABLE &&qdba..q$seg DROP CONSTRAINT q$seg_PK;

ALTER TABLE &&qdba..q$seg
   ADD CONSTRAINT q$seg_PK PRIMARY KEY 
      (type,name,schema) 
      USING INDEX
      TABLESPACE &&qtablespace
      STORAGE 
         (INITIAL     32K 
          NEXT        128K
          MINEXTENTS  1 
          PCTINCREASE 0);

ALTER TABLE &&qdba..q$seg
   ADD (file#     INTEGER
       ,block#    INTEGER
       ,watchcount INTEGER
       ,last_watch_date  DATE
       ,obj#      NUMBER);

ALTER TABLE &&qdba..q$segwatch
   ADD (file#     INTEGER
       ,block#    INTEGER);

REM *********************************************
REM initialize q$seg.watchcount to 0
REM *********************************************

UPDATE &&qdba..q$seg SET watchcount = 0;
COMMIT WORK;
