REM ******************************************************
REM File:  Q110_11A.DDL
REM
REM Makes QDBA schema modifications for migrating from
REM Q v1.1 to Q v1.1a
REM ******************************************************
REM ******************************************************
REM Revision History:
REM
REM 01/27/98: original creation                      [JMB]
REM ******************************************************

CREATE TABLE &&qdba..q$bgp_config_temp
   TABLESPACE &&qtablespace
STORAGE (INITIAL 	1 
	      NEXT 		1
         MINEXTENTS  1
 	      PCTINCREASE 	0)
AS 
   SELECT * FROM &&qdba..q$bgp_config;

UPDATE &&qdba..q$bgp_config_temp 
   SET segwatch=DECODE(segwatch,'SEGWATCH','TEMP_SEGS',segwatch);

DROP TABLE &&qdba..q$bgp_config;

CREATE TABLE &&qdba..q$bgp_config
   (id               VARCHAR2(12)   NOT NULL PRIMARY KEY 
   ,collect          VARCHAR2(12)   NOT NULL
            CONSTRAINT ck_collect 
            CHECK ( collect IN ('COLLECT','NO_COLLECT') )
   ,bench            VARCHAR2(12)   NOT NULL
            CONSTRAINT ck_bench
            CHECK ( bench IN ('BENCH','NO_BENCH') )
   ,alarm            VARCHAR2(12)   NOT NULL
            CONSTRAINT ck_alarm
            CHECK ( alarm IN ('ALARM','NO_ALARM') )
   ,segwatch         VARCHAR2(12)   NOT NULL
            CONSTRAINT ck_segwatch
            CHECK ( segwatch IN ('TEMP_SEGS','ALL_SEGS','NO_SEGWATCH') )
   ,keep_pkgs        VARCHAR2(12)   NOT NULL
            CONSTRAINT ck_keep_pkgs
            CHECK ( keep_pkgs IN ('KEEP','NO_KEEP') )
   ,collect_interval INTEGER        NOT NULL
            CONSTRAINT ck_collect_interval
            CHECK ( collect_interval > 0 )
   ,bench_interval   INTEGER        NOT NULL
            CONSTRAINT ck_bench_interval
            CHECK ( bench_interval > 0 )
   ,alarm_interval   INTEGER        NOT NULL
            CONSTRAINT ck_alarm_interval
            CHECK ( alarm_interval > 0 )
   ,detail_mode      VARCHAR2(12)   NOT NULL
            CONSTRAINT ck_detail_mode
            CHECK ( detail_mode IN ('DAILY','MANUAL','CONTINUOUS') )
   ,detail_purge     VARCHAR2(12)   NOT NULL
            CONSTRAINT ck_detail_purge
            CHECK ( detail_purge IN ('PURGE','NO_PURGE') )
   ,detail_start_date  DATE         NULL
   ,detail_end_date    DATE         NULL
   ,last_obj           INTEGER      NOT NULL
   ,last_objtype       VARCHAR2(20) NOT NULL
            CONSTRAINT ck_last_objtype
            CHECK ( last_objtype IN ('TABLE','INDEX','CLUSTER') )
   )
TABLESPACE &&qtablespace
PCTFREE	5
STORAGE (INITIAL 	1 
	      NEXT 		1
         MINEXTENTS  1
 	      PCTINCREASE 	0);

CREATE OR REPLACE TRIGGER &&qdba..q$bgp_config_UDbr 
BEFORE UPDATE OR DELETE
ON &&qdba..q$bgp_config
FOR EACH ROW
BEGIN
   IF UPDATING AND :NEW.id <> :OLD.id 
   THEN 
      raise_application_error(-20001, 'Cannot update ID');
   END IF;
   IF DELETING AND :OLD.id IN ('DEFAULT','CURRENT')
   THEN
      raise_application_error(-20001, 'Cannot DELETE id: '||:OLD.id);
   END IF;   
END;
/

INSERT INTO &&qdba..q$bgp_config
SELECT id
      ,collect
      ,bench
      ,alarm
      ,segwatch
      ,keep_pkgs
      ,collect_interval
      ,bench_interval
      ,alarm_interval
      ,detail_mode
      ,detail_purge
      ,detail_start_date
      ,detail_end_date
      ,-1
      ,'TABLE'
FROM &&qdba..q$bgp_config_temp;
COMMIT WORK;

DROP TABLE &&qdba..q$bgp_config_temp;

