/* ==================================================== */
/* Copyright(c) 1996,1997 by SAVANT Corp.               */
/* All rights reserved.                                 */
/* ==================================================== */
REM 
REM  Updgrades Beta QDBA schema to v1.0.0 schema
REM
REM ***************************************************
REM Revision History:
REM
REM 11/24/97: added PRIMARY KEY constraint to id col [JMB]
REM           of q$bgp_config
REM 11/24/97: modified insert of q$bgp_config rows  [JMB]
REM           to work if script re-executed
REM 05/29/97 added minextents 1                     [JMB]
REM 00/00/96 Original creation:                     [JMB]
REM ***************************************************

/* ---------------------------------------------------- */
/* plan_table for EXPLAIN output                        */
/* ---------------------------------------------------- */
DROP TABLE &&qdba..q$plan_table;

CREATE TABLE &&qdba..q$plan_table 
   (statement_id  VARCHAR2(30)
   ,timestamp     DATE
   ,remarks       VARCHAR2(80)
   ,operation     VARCHAR2(30)
   ,options       VARCHAR2(30)
   ,object_node   VARCHAR2(128)
   ,object_owner  VARCHAR2(30)
   ,object_name   VARCHAR2(30)
   ,object_instance  NUMERIC
   ,object_type   VARCHAR2(30)
   ,optimizer     VARCHAR2(255)
   ,search_columns   NUMERIC
   ,id            NUMERIC
   ,parent_id     NUMERIC
   ,position      NUMERIC
   ,cost          NUMERIC
   ,cardinality   NUMERIC
   ,bytes         NUMERIC
   ,other_tag     VARCHAR2(255)
   ,other         LONG
   )
TABLESPACE &&qtablespace
PCTFREE 5
STORAGE (INITIAL 	      64K 
	      NEXT 		      16K
         MINEXTENTS     1 
	      PCTINCREASE 	0);

REM DROP PUBLIC SYNONYM q$plan_table;
REM CREATE PUBLIC SYNONYM q$plan_table FOR &&qdba..q$plan_table;


/* ---------------------------------------------------- */
/* added trend columns for trending optimizatons        */
/* also modified index q$instat_log_AK1                 */
/* ---------------------------------------------------- */

ALTER TABLE &&qdba..q$instat_log ADD
   (trend_hh24    CHAR(2)
   ,trend_day     CHAR(3) );

UPDATE q$instat_log SET trend_hh24 = to_char(collect_sysdate,'HH24')
   WHERE trend_hh24 IS NULL;
commit;

UPDATE q$instat_log SET trend_day = to_char(collect_sysdate,'DY')
   WHERE trend_day IS NULL;
commit;

ALTER TABLE &&qdba..q$instat_log MODIFY
   (trend_hh24    NOT NULL 
   ,trend_day     NOT NULL );


DROP INDEX &&qdba..q$instat_log_AK1;

CREATE UNIQUE INDEX &&qdba..q$instat_log_AK1
   ON &&qdba..q$instat_log (collect_sysdate,trend_hh24,trend_day)
TABLESPACE &&qtablespace
STORAGE (INITIAL     128K 
         NEXT        64K
         MINEXTENTS  1
         PCTINCREASE 0);

/* ---------------------------------------------------- */
/* Configuraton table for Diagnostic Engine             */
/* ---------------------------------------------------- */
/* ----- config table for bgproc driver ------------*/

CREATE TABLE &&qdba..q$bgp_config
   (id               VARCHAR2(12)   NOT NULL PRIMARY KEY
   ,collect          VARCHAR2(12)   NOT NULL
            CONSTRAINT ck_collect 
            CHECK ( collect IN ('COLLECT','NO_COLLECT') )
   ,bench            VARCHAR2(12)   NOT NULL
            CONSTRAINT ck_bench
            CHECK ( bench IN ('BENCH','NO_BENCH') )
   ,alarm            VARCHAR2(12)   NOT NULL
            CONSTRAINT ck_alarm
            CHECK ( alarm IN ('ALARM','NO_ALARM') )
   ,segwatch         VARCHAR2(12)   NOT NULL
            CONSTRAINT ck_segwatch
            CHECK ( segwatch IN ('TEMP_SEGS','ALL_SEGS','NO_SEGWATCH') )
   ,keep_pkgs        VARCHAR2(12)   NOT NULL
            CONSTRAINT ck_keep_pkgs
            CHECK ( keep_pkgs IN ('KEEP','NO_KEEP') )
   ,collect_interval INTEGER        NOT NULL
            CONSTRAINT ck_collect_interval
            CHECK ( collect_interval > 0 )
   ,bench_interval   INTEGER        NOT NULL
            CONSTRAINT ck_bench_interval
            CHECK ( bench_interval > 0 )
   ,alarm_interval   INTEGER        NOT NULL
            CONSTRAINT ck_alarm_interval
            CHECK ( alarm_interval > 0 )
   ,detail_mode      VARCHAR2(12)   NOT NULL
            CONSTRAINT ck_detail_mode
            CHECK ( detail_mode IN ('DAILY','MANUAL','CONTINUOUS') )
   ,detail_purge     VARCHAR2(12)   NOT NULL
            CONSTRAINT ck_detail_purge
            CHECK ( detail_purge IN ('PURGE','NO_PURGE') )
   ,detail_start_date  DATE         NULL
   ,detail_end_date    DATE         NULL
   )
TABLESPACE &&qtablespace
PCTFREE	5
STORAGE (INITIAL 	1 
	      NEXT 		1
         MINEXTENTS  1
 	      PCTINCREASE 	0);


CREATE OR REPLACE TRIGGER &&qdba..q$bgp_config_UDbr 
BEFORE UPDATE OR DELETE
ON &&qdba..q$bgp_config
FOR EACH ROW
BEGIN
   IF UPDATING AND :NEW.id <> :OLD.id 
   THEN 
      raise_application_error(-20001, 'Cannot update ID');
   END IF;
   IF DELETING AND :OLD.id IN ('DEFAULT','CURRENT')
   THEN
      raise_application_error(-20001, 'Cannot DELETE id: '||:OLD.id);
   END IF;   
END;
/

ALTER TRIGGER &&qdba..q$bgp_config_UDbr DISABLE;

DELETE FROM &&qdba..q$bgp_config;

INSERT INTO &&qdba..q$bgp_config
   VALUES ('DEFAULT','COLLECT','NO_BENCH','NO_ALARM','TEMP_SEGS','KEEP'
                    ,600      ,600       ,300   
          ,'DAILY','PURGE',trunc(sysdate)+20/24,trunc(sysdate)+20/24);

INSERT INTO &&qdba..q$bgp_config
   VALUES ('CURRENT','COLLECT','NO_BENCH','NO_ALARM','TEMP_SEGS','KEEP'
                    ,600      ,600       ,300   
          ,'DAILY','PURGE',trunc(sysdate)+20/24,trunc(sysdate)+20/24);

COMMIT;

ALTER TRIGGER &&qdba..q$bgp_config_UDbr ENABLE;

