rem ==========================================================
rem  QRPT1A.SQL
rem
rem  Copyright(c) 1997,1998 by Savant Corporation
rem
rem ==========================================================
rem  Revision History:
rem
rem  01/05/97: added linseize 129                 [JMB]
rem  11/21/97: created                            [JMB]
rem ==========================================================
rem  NOTES:
rem 
rem  This is an example of useful reporting which can be 
rem  done against the Q Diagnostic Center's historical data.
rem 
rem  The report queries the Q$INSTAT_DLTA view and displays
rem  averages of key ORACLE performance metrics by hour of
rem  the day over the last seven days.
rem
rem  The output can be used to develop a profile of an ORACLE
rem  database's usage patterns over the course of an average 
rem  day.  Significant differences may be noticed between online
rem  usage periods and batch loads, for instance.
rem
rem  Similar reports using other performance and usage metrics 
rem  can be developed by Q users by imitating this report.
rem
rem  The report can be spooled to a file and printed landscape.
rem ============================================================

col today new_value rpt_date
set termout off
select TO_CHAR(SYSDATE,'FXMonth DD, YYYY') today from dual;
set termout on

ttitle right '&&rpt_date' skip 2 -
       center 'Hourly Performance Metrics Report' skip 1 -
       center'(previous 7 days)' skip 2
set pages 40
set linesize 129

col HOD format A7 head "Hour|of Day"
col BHR format 999.9 head "Buffer|Hit Ratio"
col DCHR format 999.9 head "Dicache| Hit Ratio"
col LCHR format 999.9 head "Libcache| Hit Ratio"
col DSPM format 9999999999 head "Disk Sorts|per Min"
col MSPM format 9999999999 head "Mem Sorts|per Min"
col PRPM format 99999999999999 head "Phys Rds|per Min"
col PWPM format 99999999999999 head "Phys Wrts|per Min"
col PCPM format 99999999999999 head "SQL Parses|per Min"
col ECPM format 99999999999999 head "SQL Execs|per Min"

SELECT
       DECODE(trend_hh24,'00','12-1am','01','1-2am','02','2-3am','03','3-4am','04','4-5am'
            ,'05','5-6am','06','6-7am','07','7-8am','08','8-9am','09','9-10am','10','10-11am'
            ,'11','11-12am','12','12-1pm','13','1-2pm','14','2-3pm','15','3-4pm','16','4-5pm'
            ,'17','5-6pm','18','6-7pm','19','7-8pm','20','8-9pm','21','9-10pm','22','10-11pm'
            ,'23','11-12pm')  HOD
      ,ROUND(AVG((1-physical_reads/(db_block_gets+consistent_gets))*100),1) BHR
      ,ROUND(AVG((1-dicache_getmisses/dicache_gets)*100),1) DCHR
      ,ROUND(AVG((libcache_gethits/libcache_gets)*100),1) LCHR
      ,ROUND(AVG((sorts_disk/collect_interval_secs)*60)) DSPM
      ,ROUND(AVG((sorts_memory/collect_interval_secs)*60)) MSPM
      ,ROUND(AVG((dbf_physrds/collect_interval_secs)*60)) PRPM
      ,ROUND(AVG((dbf_physwrts/collect_interval_secs)*60)) PWPM
      ,ROUND(AVG((parse_count/collect_interval_secs)*60)) PCPM
      ,ROUND(AVG((execute_count/collect_interval_secs)*60)) ECPM
FROM 
      q$instat_dlta
WHERE
      collect_sysdate > sysdate -7
GROUP BY trend_hh24
/

ttitle off
