/* ==================================================== */
/* Copyright(c) 1996,1997,1998 by SAVANT Corporation    */
/* All rights reserved.                                 */
/* ==================================================== */

REM ******************************************************
REM Revision history:
REM
REM 01/27/98: added GRANT w/GRANT on V_$TIMER for    [JMB]
REM           v7.2 version of Q$SESSION
REM 11/14/97: dropped grant on Q$LCKCON              [JMB]
REM 11/14/97: added GRANTs w/GRANT so GRANT to role  [JMB]
REM           will work 
REM 11/13/97: added EXECUTE on DBMS_IJOB             [JMB]
REM 10/02/97: add DROP ANY PROCEDURE so qdba can     [JMB]
REM           drop q$instl package
REM 09/30/97: add GRANT ALL on q$instl so QDBA can   [JMB]
REM           drop package later
REM 09/30/97: added numerous SYS privs to replace    [JMB]
REM           DBA role for qdba
REM 09/30/97: added SELECT ANY TABLE                 [JMB]
REM 07/07/97: added GRANT on sys.v_$sql              [JMB]
REM 06/19/97: added GRANT on sys.tab$                [JMB]
REM 06/13/97: added Q$LOCKC view for more accurate   [JMB]
REM           lock contention lists
REM 06/10/97: added dbms_* package GRANTs            [JMB]
REM 05/30/97: added GRANT on DBA_OBJECTS             [JMB]
REM 05/30/97: added Q$LOCK for new Q$LCKCON view to  [JMB]
REM           enable finding TX lock objects 
REM           on LOCK LIST
REM 00/0/96: original creation                       [JMB]
REM ******************************************************

/* ---------------------------------------------------- */
/* Q$LOCK view used for new Q$LOCKC view  below         */
/* and to enable objects on TX locks in LOCK LIST       */
/* ---------------------------------------------------- */
CREATE OR REPLACE VIEW q$lock
AS
SELECT /* HINTS REMOVED ordered use_nl(l), use_nl(s), use_nl(r) +*/  
       l.laddr       addr
      ,l.kaddr       kaddr
      ,s.ksusenum    sid
      ,s.ksuudlna    username
      ,r.ksqrsidt    type
      ,r.ksqrsid1    id1
      ,r.ksqrsid2    id2
      ,l.lmode       lmode
      ,l.request     rmode 
/* NOTE: ctime and block are 7.3 specific  */
/*    ,l.ctime       ctime  */
/*    ,l.block       block  */
      ,s.ksuseobj    obj#
      ,l.raddr       raddr
 FROM
       v$_lock l
      ,x$ksuse s
      ,x$ksqrs r
WHERE 
       l.saddr = s.addr 
  AND  l.raddr = r.addr;

/* ---------------------------------------------------- */
/* Q$LOCKC view used for newest QDBA.Q$LCKCON view      */
/* and to enable objects on TX locks in LOCK LIST       */
/* ---------------------------------------------------- */
CREATE OR REPLACE VIEW q$lockc
AS
select  lk.*
  from  q$lock  lk
       ,v$_lock r
 where 
        lk.raddr = r.raddr
   and  r.request !=0;

/* ---------------------------------------------------- */
/* Q$LATCH view of latches (required to work around     */
/* BUG# 366774 in ORACLE7.2)                            */
/* ---------------------------------------------------- */
CREATE OR REPLACE VIEW q$latch
   (addr
   ,latch#
   ,level#
   ,name
   ,gets
   ,misses
   ,sleeps
   ,immediate_gets
   ,immediate_misses
   ,waiters_woken
   ,waits_holding_latch
   ,spin_gets)
AS 
SELECT
       t.addr
      ,t.kslltnum
      ,n.kslldlvl
      ,n.kslldnam
      ,t.kslltwgt
      ,t.kslltwff
      ,t.kslltwsl
      ,t.kslltngt
      ,t.kslltnfa
      ,t.kslltwkc
      ,t.kslltwth
      ,t.ksllthst0
  FROM
       x$ksllt t
      ,x$kslld n
 WHERE
       t.kslltnum = n.indx;

/* ---------------------------------------------------- */
/* Q$BH copy of X$BH, must be created as SYS            */
/* ---------------------------------------------------- */
CREATE OR REPLACE VIEW q$bh
AS 
SELECT * FROM x$bh;

/* ---------------------------------------------------- */
/* Q$SPFREE view of free memory chunks from X$KSMSP,    */
/* must be created as SYS                               */
/* ---------------------------------------------------- */
CREATE OR REPLACE VIEW q$spfree
   (totfree_KB, maxfree_KB, free_exts) 
AS 
SELECT  
        sum(ksmchsiz)/1024  totfree_KB
       ,max(ksmchsiz)/1024  maxfree_KB
       ,count(1)            free_exts
  FROM
        x$ksmsp
 WHERE  
        ksmchcom='free memory';

/* ---------------------------------------------------- */
/* Q$KSPPI, must be created as SYS                      */
/* ---------------------------------------------------- */
CREATE OR REPLACE VIEW q$ksppi
AS
SELECT * FROM x$ksppi;


/* ---------------------------------------------------- */
/* GRANTS to QDBA                                       */
/* ---------------------------------------------------- */
GRANT ALL ON sys.q$instl TO &&qdba;

GRANT EXECUTE ON sys.dbms_lock TO &&qdba;
GRANT EXECUTE ON sys.dbms_pipe TO &&qdba;
GRANT EXECUTE ON sys.dbms_session TO &&qdba;
GRANT EXECUTE ON sys.dbms_sql TO &&qdba;
GRANT EXECUTE ON sys.dbms_shared_pool to &&qdba;
GRANT EXECUTE ON sys.dbms_job TO &&qdba;
GRANT EXECUTE ON sys.dbms_ijob TO &&qdba;
/* ---------------------------------------------------- */
/* following added 6/10/97, not sure why pkg compiles   */
/* worked before without these direct GRANTS    [JMB]   */
/* ---------------------------------------------------- */
GRANT EXECUTE ON sys.dbms_utility TO &&qdba;
GRANT EXECUTE ON sys.dbms_output TO &&qdba;
GRANT EXECUTE ON sys.dbms_space TO &&qdba;
/* ---------- end of 6/10/97 adds --------------------- */

GRANT SELECT ON q$bh TO &&qdba;
GRANT SELECT ON q$spfree TO &&qdba;
GRANT SELECT ON q$ksppi TO &&qdba;
GRANT SELECT ON q$latch TO &&qdba;
GRANT SELECT ON q$lock TO &&qdba;
GRANT SELECT ON q$lockc TO &&qdba;

GRANT SELECT ON sys.ts$ TO &&qdba;
GRANT SELECT ON sys.file$ TO &&qdba;
GRANT SELECT ON sys.seg$ TO &&qdba;
GRANT SELECT ON sys.clu$ TO &&qdba;
GRANT SELECT ON sys.fet$ TO &&qdba;
GRANT SELECT ON sys.v_$filestat TO &&qdba;
GRANT SELECT ON sys.v_$lock TO &&qdba;
GRANT SELECT ON sys.v_$session TO &&qdba;
GRANT SELECT ON sys.obj$ TO &&qdba;
GRANT SELECT ON sys.user$ TO &&qdba;
GRANT SELECT ON sys.v_$access TO &&qdba;
GRANT SELECT ON sys.user$ TO &&qdba;
GRANT SELECT ON sys.v_$sesstat TO &&qdba;
GRANT SELECT ON sys.v_$statname TO &&qdba;
GRANT SELECT ON sys.v_$sess_io TO &&qdba;
GRANT SELECT ON sys.fet$ TO &&qdba;
GRANT SELECT ON sys.file$ TO &&qdba;
GRANT SELECT ON sys.v_$db_object_cache TO &&qdba;
GRANT SELECT ON sys.v_$sgastat TO &&qdba;
GRANT SELECT ON sys.v_$waitstat TO &&qdba;
rem GRANT SELECT ON sys.v$latch TO &&qdba;
GRANT SELECT ON sys.v_$rowcache TO &&qdba;
GRANT SELECT ON sys.v_$librarycache TO &&qdba;
GRANT SELECT ON sys.v_$rollstat TO &&qdba;
GRANT SELECT ON sys.v_$system_event TO &&qdba;
GRANT SELECT ON sys.v_$log TO &&qdba;
GRANT SELECT ON sys.dba_exp_files TO &&qdba;
GRANT SELECT ON sys.v_$database TO &&qdba;
GRANT SELECT ON sys.dba_data_files TO &&qdba;
GRANT SELECT ON sys.dba_free_space TO &&qdba;
GRANT SELECT ON sys.dba_snapshots TO &&qdba;
GRANT SELECT ON sys.dba_db_links TO &&qdba;
GRANT SELECT ON sys.v_$dblink TO &&qdba;
GRANT SELECT ON sys.v_$filestat TO &&qdba;
GRANT SELECT ON sys.v_$parameter TO &&qdba;
GRANT SELECT ON sys.v_$sysstat TO &&qdba;
GRANT SELECT ON sys.v_$datafile TO &&qdba;
GRANT SELECT ON sys.v_$instance TO &&qdba;
GRANT SELECT ON sys.v_$sga TO &&qdba;
GRANT SELECT ON sys.v_$latchname TO &&qdba;
GRANT SELECT ON sys.dba_segments TO &&qdba;
GRANT SELECT ON sys.dba_users TO &&qdba;
GRANT SELECT ON sys.dba_profiles TO &&qdba;
GRANT SELECT ON sys.sysauth$ TO &&qdba;
GRANT SELECT ON sys.v_$timer TO &&qdba;
GRANT SELECT ON sys.sys_objects TO &&qdba;
GRANT SELECT ON sys.link$ TO &&qdba;
GRANT SELECT ON sys.ind$ TO &&qdba;
GRANT SELECT ON sys.trigger$ TO &&qdba;
GRANT SELECT ON sys.dba_ind_columns TO &&qdba;
GRANT SELECT ON sys.dba_synonyms TO &&qdba;
GRANT SELECT ON sys.undo$ TO &&qdba;
GRANT SELECT ON sys.v_$transaction TO &&qdba;
GRANT SELECT ON sys.v_$sqlarea TO &&qdba;
GRANT SELECT ON sys.v_$open_cursor TO &&qdba;
GRANT SELECT ON sys.defrole$ TO &&qdba;
GRANT SELECT ON sys.syn$ TO &&qdba;
GRANT SELECT ON sys.v_$sqltext TO &&qdba;
GRANT SELECT ON sys.v_$sql TO &&qdba;
GRANT SELECT ON sys.v_$rollname TO &&qdba;
GRANT SELECT ON sys.v_$process TO &&qdba;
GRANT SELECT ON sys.v_$session_wait TO &&qdba;
GRANT SELECT ON sys.v_$latchholder TO &&qdba;
GRANT SELECT ON sys.v_$session_event TO &&qdba;

GRANT SELECT ON sys.v_$dispatcher TO &&qdba;
GRANT SELECT ON sys.v_$queue TO &&qdba;
GRANT SELECT ON sys.v_$shared_server TO &&qdba;
GRANT SELECT ON sys.v_$circuit TO &&qdba;
GRANT SELECT ON sys.v_$bgprocess TO &&qdba;

GRANT SELECT ON sys.dba_jobs_running TO &&qdba;
GRANT SELECT ON sys.dba_jobs TO &&qdba;

GRANT SELECT ON sys.dba_objects TO &&qdba;
GRANT SELECT ON sys.tab$ TO &&qdba;

rem ***************************************************
rem added privs so DBA role could be removed
rem ***************************************************
GRANT CREATE SESSION TO &&qdba;
GRANT ANALYZE ANY TO &&qdba;
GRANT BECOME USER TO &&qdba;
GRANT ALTER SYSTEM TO &&qdba;
GRANT ALTER SESSION TO &&qdba;
GRANT CREATE SEQUENCE TO &&qdba;
GRANT CREATE TABLE TO &&qdba;
GRANT CREATE CLUSTER TO &&qdba;
GRANT CREATE VIEW TO &&qdba;
GRANT CREATE SYNONYM TO &&qdba;
GRANT CREATE PUBLIC SYNONYM TO &&qdba;
GRANT CREATE PROCEDURE TO &&qdba;
GRANT CREATE TRIGGER TO &&qdba;
GRANT SELECT ANY TABLE TO &&qdba;
GRANT CREATE ROLE TO &&qdba;
GRANT DROP ANY PROCEDURE TO &&qdba;

REM ****************************************
REM GRANT WITH GRANT to QDBA so
REM secondary VIEW GRANTs will work
REM ****************************************
GRANT SELECT ON SYS.V_$ACCESS TO &&qdba WITH GRANT OPTION;
GRANT SELECT ON SYS.V_$FILESTAT  TO &&qdba WITH GRANT OPTION;
GRANT SELECT ON SYS.V_$SESSION TO &&qdba WITH GRANT OPTION;
GRANT SELECT ON SYS.V_$DB_OBJECT_CACHE TO &&qdba WITH GRANT OPTION;
GRANT SELECT ON SYS.V_$BGPROCESS TO &&qdba WITH GRANT OPTION;
GRANT SELECT ON SYS.V_$STATNAME TO &&qdba WITH GRANT OPTION;
GRANT SELECT ON SYS.V_$SESS_IO TO &&qdba WITH GRANT OPTION;
GRANT SELECT ON SYS.V_$CIRCUIT TO &&qdba WITH GRANT OPTION;
GRANT SELECT ON SYS.V_$SESSTAT TO &&qdba WITH GRANT OPTION;
GRANT SELECT ON SYS.V_$TIMER TO &&qdba WITH GRANT OPTION;
GRANT SELECT ON SYS.Q$LOCKC TO &&qdba WITH GRANT OPTION;
GRANT SELECT ON SYS.TS$ TO &&qdba WITH GRANT OPTION;
GRANT SELECT ON SYS.FILE$ TO &&qdba WITH GRANT OPTION;
GRANT SELECT ON SYS.FET$ TO &&qdba WITH GRANT OPTION;
GRANT SELECT ON SYS.CLU$ TO &&qdba WITH GRANT OPTION;
GRANT SELECT ON SYS.SEG$ TO &&qdba WITH GRANT OPTION;

REM ****************************************
REM GRANT SELECT on Q views to Q_USER_ROLE
REM ****************************************
GRANT SELECT ON Q$BH TO Q_USER_ROLE;
GRANT SELECT ON Q$KSPPI TO Q_USER_ROLE;
GRANT SELECT ON Q$LATCH TO Q_USER_ROLE;
rem GRANT SELECT ON Q$LCKCON TO Q_USER_ROLE;
GRANT SELECT ON Q$LOCK TO Q_USER_ROLE;
GRANT SELECT ON Q$LOCKC TO Q_USER_ROLE;
GRANT SELECT ON Q$SPFREE TO Q_USER_ROLE;

REM *****************************************
REM system privs for Q_USER_ROLE
REM *****************************************
GRANT ALTER SESSION, BECOME USER, SELECT ANY TABLE TO Q_USER_ROLE;
