REM --------------------------------------------------------------
REM   Qtfail1.sql   
REM
REM   Copyright(c) 1997,1998 by SAVANT Corp.
REM   All rights reserved.             
REM --------------------------------------------------------------
REM ==============================================================
REM  SQL*Plus report providing details for a segment predicted 
REM  by Q Diagnostic Engine to be in danger of failing due to 
REM  maxextents or free extent too small in tablespace.
REM
REM  Command line parameters:
REM     1)  tablespace name
REM     2)  segment schema
REM     3)  segment name
REM
REM  Revision History:
REM
REM  12/19/97: re-written to display results using    [JMB]
REM            prompt rather than DBMS_OUTPUT
REM  12/16/97: original creation                      [JMB]
REM ==============================================================
define tablespacename = &1
define schema = &2
define segment_name = &3

ALTER SESSION SET NLS_DATE_FORMAT='MM/DD/YY HH24:MI:SS';

set verify off
set feedback off
set termout off
set numwidth 12
set serveroutput OFF

var db_block_size NUMBER
BEGIN
   :db_block_size := q$instat.parmval('db_block_size');
END;
/

col segname new_value seg_name
col segschema new_value segment_schema
col segtype new_value segment_type
col extents new_value seg_extents
col nextext new_value next_extent_KB
col used new_value used_KB
col free new_value free_KB
col KB_pmin new_value KB_per_min
col calc_date new_value calculate_date
col fail_date new_value failure_date
col fail_reason new_value failure_reason
col init_extent new_value initial_extent_KB
col max_exts new_value max_extents format 9999999999999
col pctincrease new_value pct_increase
col tbsname new_value tablespace_name
col tbsfree_KB new_value tbs_free_KB
col tbsmaxfree_KB new_value tbs_maxfree_KB

SELECT  S.name                                segname
       ,S.schema                              segschema
       ,S.type                                segtype
       ,SG.extents                            extents
       ,SG.extsize * :db_block_size/1024      nextext
       ,S.used_blks * :db_block_size/1024     used              
       ,S.free_blks * :db_block_size/1024     free
       ,ROUND(S.blks_per_min * :db_block_size/1024,1)
                                              KB_pmin
       ,S.calc_date                           calc_date
       ,S.fail_date                           fail_date
       ,S.fail_reason                         fail_reason
       ,SG.iniexts * :db_block_size/1024      init_extent
       ,SG.maxexts                            max_exts
       ,SG.extpct                             pctincrease
       ,TS.name                               tbsname 
       ,TF.free_blks * :db_block_size/1024    tbsfree_KB
       ,TF.maxfree_blks * :db_block_size/1024 tbsmaxfree_KB
  FROM  
        q$seg    S
       ,sys.ts$  TS
       ,sys.seg$ SG
       ,q$tbsfree  TF
 WHERE 
       S.schema = UPPER('&&schema')
   AND S.name   = UPPER('&&segment_name')
   AND S.ts#  = TS.ts#
   AND TF.ts# = TS.ts#
   AND SG.file# = S.file#
   AND SG.block# = S.block#
   AND SG.ts#  = S.ts#;

set termout on
set feedback on
set verify on
set pages 100

prompt 
prompt : ===============================================================
prompt :             Segment Failure Details Report               
prompt : ===============================================================
prompt :    
prompt :     &&segment_schema..&&seg_name
prompt :     ========================================
prompt :                type: &&segment_type 
prompt :          tablespace: &&tablespace_name
prompt :    
prompt :    
prompt :     Storage
prompt :     ========================================
prompt :             initial: &&initial_extent_KB KB
prompt :                next: &&next_extent_KB KB
prompt :            pct incr: &&pct_increase
prompt :         max extents: &&max_extents
prompt :        curr extents: &&seg_extents
prompt :    
prompt :     Tablespace Free Space Info
prompt :     ========================================
prompt :      max free size: &&tbs_maxfree_KB KB
prompt :         total free: &&tbs_free_KB KB
prompt :    
prompt :    
prompt :    
prompt :     Growth Info
prompt :     ========================================
prompt :          calc_date: &&calculate_date    
prompt :        growth rate: &&KB_per_min KB per min
prompt :         used space: &&used_KB KB
prompt :         free space: &&free_KB KB
prompt :        fail reason: &&failure_reason
prompt :          fail date: &&failure_date
prompt :    
prompt : ===============================================================
