/* ==================================================== */
/* Copyright(c) 1996,1997 by SAVANT Corp.               */
/* All rights reserved.                                 */
/* ==================================================== */

REM *****************************************************
REM Revision history:
REM
REM 10/29/97: merged with Oracle8 modified version [JMB]
REM           of QVIEWS.DDL
REM 10/27/97: modified Q$INSTANCE                  [JMB]
REM 10/27/97: creation cloned from QVIEWS73.DDL    [JMB]
REM *****************************************************

/* ---------------------------------------------------- */
/* Q$INSTANCE                                           */
/*                                                      */
/* Used to instantiate startup_time consistent with     */
/* Oracle8 changes to V$INSTANCE                        */
/* ---------------------------------------------------- */
CREATE OR REPLACE VIEW q$instance
AS
SELECT  *
  FROM  v$instance;

/* ---------------------------------------------------- */
/* Q$ACCESS                                             */
/*                                                      */
/* Creates a view which looks similar to V$ACCESS under */
/* to mask 7.2 to 7.3 differences                       */
/* ---------------------------------------------------- */
CREATE OR REPLACE VIEW q$access
   (sid
   ,owner
   ,object
   ,type )
AS
SELECT
       sid
      ,owner
      ,object
      ,type
 FROM 
       sys.v_$access;

/* ---------------------------------------------------- */
/* Q$SESSION = create a view called Q$SESSION for       */ 
/*             V$SESSION                                */
/* ---------------------------------------------------- */

CREATE OR REPLACE VIEW Q$SESSION AS 
   select s.*,p.paddr bg_paddr 
     from sys.v_$session s
          ,sys.v_$bgprocess p
    where s.paddr = p.paddr(+);

REM ****************************************************
REM Revision history:
REM
REM 07/18/97: ORACLE8 
REM 06/13/97: newer Q$LCKCON view using new     [JMB]
REM           SYS view Q$LOCKC
REM 05/30/97: new Q$LCKCON view                 [JMB]
REM 00/00/96: original creation                 [JMB]
REM ****************************************************

/* ---------------------------------------------------- */
/* Q$INSTAT_DLTA creates delta stats on which trends    */
/* are computed                                         */
/* ---------------------------------------------------- */
CREATE OR REPLACE VIEW &&qdba..q$instat_dlta
      (collect_seq
      ,collect_sysdate
      ,collect_interval_secs
      ,collect_hsecs
      ,inst_start_date
      ,log_sequence
      ,logons
      ,logons_cum
      ,open_cursors
      ,open_cursors_cum
      ,commits
      ,db_block_gets
      ,consistent_gets
      ,physical_reads
      ,buffer_busy_waits
      ,buffer_busy_waittime
      ,enqueue_requests
      ,enqueue_timeouts
      ,redo_size
      ,user_calls
      ,parse_count
      ,execute_count
      ,recursive_calls
      ,sorts_disk
      ,sorts_memory
      ,table_scan_rows_gotten
      ,table_fetch_rowid
      ,table_scans_short
      ,table_scans_long
      ,table_fetch_cont_row
      ,rbs_waits
      ,rbs_gets
      ,rbs_xacts
      ,latch_gets
      ,latch_misses
      ,latch_sleeps
      ,latch_imm_gets
      ,dbfile_waits
      ,dbfile_waittime
      ,lgfile_waits
      ,lgfile_waittime
      ,dicache_gets
      ,dicache_getmisses
      ,libcache_gets
      ,libcache_gethits
      ,libcache_pins
      ,libcache_pinhits
      ,libcache_reloads
      ,shpool_free_KB
      ,shpool_maxfree_KB
      ,shpool_free_exts
      ,IOskew
      ,dbf_physrds
      ,dbf_physwrts
      ,active_tempsegs
/* new MTS related statistics   */
      ,disp_busy_hsec
      ,disp_idle_hsec
      ,disp_waittime_hsec
      ,disp_totalqueue
      ,serv_busy_hsec
      ,serv_idle_hsec
      ,serv_waittime_hsec
      ,serv_totalqueue
/*  added for v1.0 trend optimizations */
      ,trend_hh24
      ,trend_day
      )
AS
select 
       A.collect_seq
      ,A.collect_sysdate
      ,(A.collect_sysdate - B.collect_sysdate) * 86400
      ,A.collect_hsecs
      ,A.inst_start_date
      ,A.log_sequence         -  B.log_sequence
      ,GREATEST(A.logons,B.logons)
      ,A.logons_cum           -  B.logons_cum
      ,GREATEST(A.open_cursors,B.open_cursors)
      ,A.open_cursors_cum     -  B.open_cursors_cum
      ,A.commits              -  B.commits
      ,A.db_block_gets        -  B.db_block_gets
      ,A.consistent_gets      -  B.consistent_gets
      ,A.physical_reads       -  B.physical_reads
      ,A.buffer_busy_waits    -  B.buffer_busy_waits
      ,A.buffer_busy_waittime -  B.buffer_busy_waittime
      ,A.enqueue_requests     -  B.enqueue_requests
      ,A.enqueue_timeouts     -  B.enqueue_timeouts
      ,A.redo_size            -  B.redo_size
      ,A.user_calls           -  B.user_calls
      ,A.parse_count          -  B.parse_count
      ,A.execute_count        -  B.execute_count
      ,A.recursive_calls      -  B.recursive_calls
      ,A.sorts_disk           -  B.sorts_disk
      ,A.sorts_memory         -  B.sorts_memory
      ,A.table_scan_rows_gotten  -  B.table_scan_rows_gotten
      ,A.table_fetch_rowid    -  B.table_fetch_rowid
      ,A.table_scans_short    -  B.table_scans_short
      ,A.table_scans_long     -  B.table_scans_long
      ,A.table_fetch_cont_row -  B.table_fetch_cont_row
      ,A.rbs_waits            -  B.rbs_waits
      ,A.rbs_gets             -  B.rbs_gets
      ,GREATEST(A.rbs_xacts,B.rbs_xacts)
      ,A.latch_gets           -  B.latch_gets
      ,A.latch_misses         -  B.latch_misses
      ,A.latch_sleeps         -  B.latch_sleeps
      ,A.latch_imm_gets       -  B.latch_imm_gets
      ,A.dbfile_waits         -  B.dbfile_waits
      ,A.dbfile_waittime      -  B.dbfile_waittime
      ,A.lgfile_waits         -  B.lgfile_waits 
      ,A.lgfile_waittime      -  B.lgfile_waittime
      ,A.dicache_gets         -  B.dicache_gets
      ,A.dicache_getmisses    -  B.dicache_getmisses
      ,A.libcache_gets        -  B.libcache_gets
      ,A.libcache_gethits     -  B.libcache_gethits
      ,A.libcache_pins        -  B.libcache_pins
      ,A.libcache_pinhits     -  B.libcache_pinhits
      ,A.libcache_reloads     -  B.libcache_reloads
      ,(A.shpool_free_KB      +  B.shpool_free_KB)/2
      ,(A.shpool_maxfree_KB   +  B.shpool_maxfree_KB)/2
      ,(A.shpool_free_exts    +  B.shpool_free_exts)/2
      ,(A.IOskew              +  B.IOskew)/2
      ,A.dbf_physrds          -  B.dbf_physrds
      ,A.dbf_physwrts         -  B.dbf_physwrts
      ,GREATEST(A.active_tempsegs,B.active_tempsegs)
/*          new MTS related statistics                   */
      ,A.disp_busy_hsec       -  B.disp_busy_hsec
      ,A.disp_idle_hsec       -  B.disp_idle_hsec
      ,A.disp_waittime_hsec   -  B.disp_waittime_hsec
      ,A.disp_totalqueue      -  B.disp_totalqueue
      ,A.serv_busy_hsec       -  B.serv_busy_hsec
      ,A.serv_idle_hsec       -  B.serv_idle_hsec
      ,A.serv_waittime_hsec   -  B.serv_waittime_hsec
      ,A.serv_totalqueue      -  B.serv_totalqueue
/*  added with v1.0 trend optimizations  */
      ,A.trend_hh24
      ,A.trend_day
from
       q$instat_log  A
      ,q$instat_log  B
where
        A.inst_start_date  = B.inst_start_date
  and   B.collect_seq      = A.collect_seq -1
  and   B.collect_sysdate  < A.collect_sysdate;


/* ---------------------------------------------------- */
/* Q$CIRCUIT MTS virtual circuits joined to sessions    */
/* ---------------------------------------------------- */
CREATE OR REPLACE VIEW &&qdba..q$circuit
AS
SELECT
       c.circuit        circuit
      ,c.dispatcher     dispatcher
      ,c.server         server
      ,c.waiter         wait_server
      ,c.status         status
      ,c.queue          queue
      ,c.messages       messages
      ,s.sid            sess_sid
      ,s.username       sess_username
  FROM
        sys.v_$circuit     c
       ,sys.v_$session     s
 WHERE
        c.saddr      =    s.saddr;

/* ---------------------------------------------------- */
/* Q$OBCHSUM Object cache summary stats                 */
/* ---------------------------------------------------- */
CREATE OR REPLACE VIEW &&qdba..q$objchsum
AS
SELECT	 namespace		namespace
	,count(1)		obj_count	
	,sum(sharable_mem)	shmem
	,sum(loads)		loads
	,sum(executions)	execs
	,sum(locks)		locks
	,sum(pins)		pins
FROM
	 sys.v_$db_object_cache
GROUP BY namespace;

/* ---------------------------------------------------- */
/* Q$TBSSEG                                             */
/* ---------------------------------------------------- */
CREATE OR REPLACE VIEW &&qdba..q$tbsseg
      (ts#, clust_ind, tbl_ind, indx_ind, rbs_ind)
AS
SELECT 
        ts.ts#                      ts#
       ,decode(sum(decode(cl.ts#,null,0,1)),0,'FALSE','TRUE') clust_ind              
       ,decode(sum(decode(sg.type#,5,1,0)),0,'FALSE','TRUE')  tbl_ind
       ,decode(sum(decode(sg.type#,6,1,0)),0,'FALSE','TRUE')  indx_ind
       ,decode(sum(decode(sg.type#,1,1,0)),0,'FALSE','TRUE')  rbs_ind
  FROM
        sys.ts$            ts
       ,sys.file$          fl
       ,sys.seg$           sg
       ,sys.clu$           cl
 WHERE
        ts.ts#       =     fl.ts#
   AND  sg.file#(+)  =     fl.file#
   AND  cl.file#(+)  =     fl.file#
   AND  ts.online$  !=     3
 GROUP BY ts.ts#;

/* ---------------------------------------------------- */
/* Q$FILFREE                                            */
/* ---------------------------------------------------- */
CREATE OR REPLACE VIEW &&qdba..q$filfree
   (file#
   ,free_bytes
   ,maxfree_bytes)
AS
   SELECT   
           file#                         file#
          ,sum(ts.blocksize*fe.length)   free_bytes
          ,max(ts.blocksize*fe.length)   maxfree_bytes
     FROM  
           sys.fet$         fe
          ,sys.ts$          ts
    WHERE  
           fe.ts#      =    ts.ts#
    GROUP BY file#;

/* ---------------------------------------------------- */
/* Q$AVG_IO                                             */
/* ---------------------------------------------------- */
CREATE OR REPLACE VIEW &&qdba..q$avg_io  (avg_io)
AS
   SELECT  sum(phyrds+phywrts)/count(*)  avg_io 
     FROM  sys.v_$filestat;


/* ------------------------------------------------- */
/* Q$LCKCON                                          */
/*                                                   */
/* Lock Contention: all locks with nonzero requests  */
/* ------------------------------------------------- */

CREATE OR REPLACE VIEW &&qdba..q$lckcon 
   (lk_usrname
   ,lk_sessid
   ,lk_req_sessid    
   ,lk_type    
   ,lk_heldmode  
   ,lk_requestmode 
   ,lk_resource1   
   ,lk_resource2
   ,lk_obj#
   )
AS 
select DISTINCT
       l1.username 
      ,l1.sid
      ,l2.sid
      ,decode(l1.type, 
         'MR', 'Media Recovery', 
         'RT', 'Redo Thread',
         'UN', 'User Name',
         'TX', 'Transaction',
         'TM', 'DML',
         'UL', 'PL/SQL User Lock',
         'DX', 'Distributed Xaction',
         'CF', 'Control File',
         'IS', 'Instance State',
         'FS', 'File Set',
         'IR', 'Instance Recovery',
         'ST', 'Disk Space Transaction',
         'TS', 'Temp Segment',
         'IV', 'Library Cache Invalidation',
         'LS', 'Log Start or Switch',
         'RW', 'Row Wait',
         'SQ', 'Sequence Number',
         'TE', 'Extend Table',
         'TT', 'Temp Table',
          l1.type)                   lk_type
      ,decode(l1.lmode, 
          0, 'None',           
          1, 'Null',          
          2, 'Row-S (SS)',    
          3, 'Row-X (SX)',     
          4, 'Share',          
          5, 'S/Row-X (SSX)',  
          6, 'Exclusive',      
          to_char(l1.lmode))       lk_heldmode
      ,decode(l1.rmode,
          0, 'None',           
          1, 'Null',          
          2, 'Row-S (SS)',     
          3, 'Row-X (SX)',     
          4, 'Share',          
          5, 'S/Row-X (SSX)',  
          6, 'Exclusive',      
          to_char(l1.rmode))       lk_requestmode     
      ,to_char(l1.id1)             lk_resource1
      ,to_char(l1.id2)             lk_resource2
      ,decode(l1.type,'TX',decode(l1.rmode,0,l2.obj#,l1.obj#)
                     ,'TM',l1.id1,decode(l2.type,'TM',l2.id1, null ) )  obj#
from 
      sys.q$lockc        l1
     ,sys.q$lockc        l2
 where 
       l1.raddr  = l2.raddr
  and  l1.lmode != l2.lmode;

/* ------------------------------------------------- */
/* Q$SESSTAT                                         */
/*                                                   */
/* Session Statistics: key stats by session id       */
/* ------------------------------------------------- */
CREATE OR REPLACE VIEW &&qdba..q$sesstat
       (sess_id
       ,sess_cpu
       ,sess_callcpu
       ,sess_connect_hsecs
       ,sess_noidle_hsecs
       ,sess_disksorts
       )
AS
SELECT
        s.sid
       ,sum(decode(sn.name,'CPU used by this session',st.value,0))
       ,sum(decode(sn.name,'CPU used when call started',st.value,0))
       ,sum(decode(sn.name,'session connect time',st.value,0))
       ,sum(decode(sn.name,'process last non-idle time',st.value,0))
       ,sum(decode(sn.name,'sorts (disk)',st.value,0))
  FROM
        sys.v_$sesstat       st
       ,sys.v_$statname      sn
       ,sys.v_$session       s
 WHERE
        sn.statistic#   =   st.statistic#
   AND  s.sid           =   st.sid
   AND  s.type          =   'USER'
   AND  s.status       !=   'KILLED'
 GROUP BY S.sid;

/* ------------------------------------------------- */
/* Q$SESSTOT                                         */
/*                                                   */
/* Session Statistics: key stats totalled            */
/* ------------------------------------------------- */
CREATE OR REPLACE VIEW &&qdba..q$sesstot
   (totcpu, totdisksorts)
AS
SELECT 
       sum(sess_cpu)        totcpu
      ,sum(sess_disksorts)  totdisksorts
  FROM 
        q$sesstat;

/* ------------------------------------------------- */
/* Q$SESSIO                                          */
/*                                                   */
/* Session IO statistics totals                      */
/* ------------------------------------------------- */
CREATE OR REPLACE VIEW &&qdba..q$sessio
   (totphysrds, totblkchgs, totblkgets, totcongets)
AS
SELECT
        sum(IO.physical_reads)  totphysrds
       ,sum(IO.block_changes)   totblkchgs
       ,sum(IO.block_gets)      totblkgets
       ,sum(IO.consistent_gets) totcongets
 FROM
       sys.v_$sess_io   IO
      ,sys.v_$session   S
 WHERE  
       S.sid       =   IO.sid
  AND  S.type      =   'USER'
  AND  S.status   !=   'KILLED';


/* ------------------------------------------------- */
/* Q$TBSFREE                                         */
/*                                                   */
/* Tablespace total and max free blocks by ts#       */
/* ------------------------------------------------- */
CREATE OR REPLACE VIEW &&qdba..q$tbsfree
       (ts#, free_blks, maxfree_blks)
AS
SELECT     
           ts#
          ,sum(length)
          ,max(length)
  FROM       
           sys.fet$
 GROUP BY ts#;


/* ------------------------------------------------- */
/* Q$TBSBLKS                                         */
/*                                                   */
/* Total tablespace size in blocks                   */
/* ------------------------------------------------- */
CREATE OR REPLACE VIEW &&qdba..q$tbsblks
      (ts#, tot_blks)
AS
SELECT
           ts#
          ,sum(blocks)
  FROM
           sys.file$
 WHERE
           status$   != 1
 GROUP BY ts#;



