REM   ----------------------------------------------------
REM   Recompil.sql   
REM
REM   Copyright(c) 1996,1997,1998 by SAVANT Corp.
REM   All rights reserved.             
REM   ----------------------------------------------------
REM
REM  This script prompts for the Q installation directory
REM  and recompiles Q packages from source code.  This is 
REM  usually only necessary when the Q Diagnostic Engine
REM  install fails with ORA-4031 or another error which 
REM  keeps the package source from getting into the 
REM  catalog.
REM
REM  If package source is in the catalog, then Q packages
REM  can be recompiled using DBMS_UTILITY from SQL*Plus
REM  as follows:
REM
REM  SQL> execute DBMS_UTILITY.COMPILE_SCHEMA('QDBA');
REM
REM
/* ==================================================== */
/*            Revision History                          */ 
/* ==================================================== */
--  11/24/97: modified for new file names (O8)   [JMB]
--  00/00/96: original creation                  [JMB]
/* ==================================================== */
set serveroutput on size 100000
set verify off

undefine install_dir

prompt ******************************************************* 
prompt This script should be run from the QDBA user account.
prompt
prompt You will be prompted for the Q installation directory
prompt which is normally C:\SAVANT for most standard installs.
prompt
prompt *******************************************************
prompt
var abort NUMBER;
BEGIN
   :abort := 0;
   IF USER != 'QDBA' 
   THEN
      :abort := :abort +1;
      dbms_output.put_line('ERROR: not connected as QDBA!  '||
                           'Script will abort in 10 seconds.');
      dbms_output.put_line('Please connect as QDBA and re-execute.');
   END IF;
END;
/
WHENEVER SQLERROR EXIT
BEGIN
   IF :abort > 0
   THEN
      dbms_lock.sleep(10);
      raise_application_error(-20000, 'Script abort detected');
   END IF;
END;
/
WHENEVER SQLERROR CONTINUE

accept install_dir CHAR prompt 'Enter Q install directory:'

undefine O7ver
set termout off
col O7 new_value O7ver
SELECT TRANSLATE(SUBSTR(version,1,3),'~.','~') O7
  FROM product_component_version
 WHERE UPPER(product) LIKE 'ORACLE7%'
    OR UPPER(product) LIKE 'PERSONAL ORACLE7%'
    OR UPPER(product) LIKE 'ORACLE8%';
set termout on

execute qdba.q$bgproc.kill_driver;

ALTER SYSTEM FLUSH SHARED_POOL;
/

set feedback off
spool &&install_dir\qsql\temp.sql
BEGIN
   IF &&O7ver IN ('72','73')
   THEN
      dbms_output.put_line('prompt Creating Q package specifications...');
      dbms_output.put_line('start &&install_dir\qsql\qdecl.wps');
      dbms_output.put_line('start &&install_dir\qsql\qcvar.wps');
      dbms_output.put_line('start &&install_dir\qsql\qinstat.wps');
      dbms_output.put_line('start &&install_dir\qsql\qbgproc.wps'); 
      dbms_output.put_line('start &&install_dir\qsql\qsess.wps');
      dbms_output.put_line('start &&install_dir\qsql\qinst2.wps');
      dbms_output.put_line('start &&install_dir\qsql\qtrend.wps');
      dbms_output.put_line('start &&install_dir\qsql\qspace.wps');
      dbms_output.put_line('start &&install_dir\qsql\qrepl.wps');
      dbms_output.put_line('start &&install_dir\qsql\qora.wps');
      dbms_output.put_line('start &&install_dir\qsql\qdetl&&O7ver..wps');
      --
      dbms_output.put_line('prompt Creating Q package bodies...');
      dbms_output.put_line('start &&install_dir\qsql\qdecl.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qcvar.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qinstat.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qbgproc.wpb'); 
      dbms_output.put_line('start &&install_dir\qsql\qsess.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qinst2.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qtrend.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qspace.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qrepl.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qora.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qdetl&&O7ver..wpb');

   ELSIF &&O7ver IN ('80','81')
   THEN
      dbms_output.put_line('prompt Creating Q package specifications...');
      dbms_output.put_line('start &&install_dir\qsql\qdecl80.wps');
      dbms_output.put_line('start &&install_dir\qsql\qcvar80.wps');
      dbms_output.put_line('start &&install_dir\qsql\qinst80.wps');
      dbms_output.put_line('start &&install_dir\qsql\qbgp80.wps'); 
      dbms_output.put_line('start &&install_dir\qsql\qsess80.wps');
      dbms_output.put_line('start &&install_dir\qsql\qinst280.wps');
      dbms_output.put_line('start &&install_dir\qsql\qtrend80.wps');
      dbms_output.put_line('start &&install_dir\qsql\qspace80.wps');
      dbms_output.put_line('start &&install_dir\qsql\qrepl80.wps');
      dbms_output.put_line('start &&install_dir\qsql\qora80.wps');
      dbms_output.put_line('start &&install_dir\qsql\qdetl80.wps');
      --
      dbms_output.put_line('prompt Creating Q package bodies...');
      dbms_output.put_line('start &&install_dir\qsql\qdecl80.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qcvar80.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qinst80.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qbgp80.wpb'); 
      dbms_output.put_line('start &&install_dir\qsql\qsess80.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qinst280.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qtrend80.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qspace80.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qrepl80.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qora80.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qdetl80.wpb');
   END IF;
END;
/
spool off
set feedback on

spool &&install_dir\qsql\recompil.log
start &&install_dir\qsql\temp.sql

execute q$bgproc.start_driver;

spool off
