REM   --------------------------------------------------------------------------
REM   Qchkall.sql   
REM
REM   Copyright(c) 1996,1997,1998 by SAVANT Corp.
REM   All rights reserved.             
REM   ---------------------------------------------------------------------------
REM ==============================================================
REM Revision History:
REM
REM 12/19/97: added ERROR: keyword when pkg invalid      [JMB]
REM 11/26/97: added text describing where DBMS create    [JMB]
REM           scripts are located and example
REM 11/26/97: added segment-specific correction msgs     [JMB]
REM           for seg next > SYSTEM maxfree error
REM 11/26/97: added check of TIMED_STATISTICS parm       [JMB]
REM 11/26/97: added WHENEVER OSERROR EXIT to abort if    [JMB]
REM           q$instl package files not found
REM 10/30/97: remove kill_driver, was already doing kill [JMB]
REM           inside q$instl.stop_driver
REM 10/29/97: calling version-specific qinstl.wps/wpb    [JMB]
REM 10/27/97: using kill_driver to stop DE (beta compat) [JMB]
REM 10/27/97: added Oracle8 as valid version             [JMB]
REM 10/27/97: moved check for Oracle version before      [JMB]
REM           creating q$instl
REM 07/02/97: better space checking on SYSTEM tablespace [JMB]
REM 07/01/97: added check for owner='SYS' to initial     [JMB]
REM           package validation (customer bug)
REM 00/00/96: original creation                          [JMB]
REM ==============================================================
undefine source_dir
undefine parm_dir
set termout on

DEFINE parm_dir=&1

rem  *** remove trailing slash from directory name
rem  *** and put into source_dir variable

col parm_dir_noslash new_value source_dir
select DECODE(
               SUBSTR('&&parm_dir',LENGTH('&&parm_dir'),1) 
               ,'/', SUBSTR('&&parm_dir',1,LENGTH('&&parm_dir')-1 )
               ,'\', SUBSTR('&&parm_dir',1,LENGTH('&&parm_dir')-1 )
               ,'&&parm_dir')            parm_dir_noslash
from dual;

DEFINE separator='===================================================================='

set serveroutput on size 1000000
set verify off
set feedback off

var check_return_code   NUMBER
var abort               NUMBER
var pkg_count           NUMBER

BEGIN
   :check_return_code := 0;
   :abort      := 0;
   :pkg_count  := 0;
END;
/

prompt &&separator
prompt QCHKALL.SQL
prompt
prompt This script checks all ORACLE requirements for installing the Q Diagnostic 
prompt Engine.  It will report any problems using the keyword "ERROR:" and these
prompt must be corrected before the Q installation can proceed.
prompt
prompt In some cases, recommendations are made for modifications to init.ora
prompt parameters.  Recommended changes are not required for installation.
prompt
prompt The script must be run as SYS.  It creates a package called Q$INSTL in the
prompt SYS schema with procedures and functions which help the checking process.
prompt This package is dropped at the end of Q installation.
prompt
prompt 
prompt &&separator

set head off
REM *************************************************************************
prompt &&separator
prompt Checking if connected as SYS...
select decode(USER,'SYS'
         ,'OK, connected as SYS.'
         ,'ERROR: Not conected as SYS!')
  from dual;

BEGIN
   IF USER != 'SYS'
   THEN
      :abort := :abort +1;
   END IF;
END;
/

REM *************************************************************************
prompt &&separator
prompt Checking ORACLE version...
prompt
define O7ver = 'XX'
var abort number
col O7 new_value O7ver

SELECT TRANSLATE(SUBSTR(version,1,3),'~.','~') O7
  FROM product_component_version
 WHERE UPPER(product) LIKE 'ORACLE7%'
    OR UPPER(product) LIKE 'PERSONAL ORACLE7%'
    OR UPPER(product) LIKE 'ORACLE8%';

BEGIN
  IF '&&O7ver' NOT IN ('72','73','80')
  THEN
      :abort := :abort +1;
      dbms_output.put_line
         ('ERROR: ORACLE version is not 7.2, 7.3 or 8.0, cannot install Q.');
   ELSE
      dbms_output.put_line('ORACLE version is OK.');
  END IF;
END;
/
prompt &&separator


REM *************************************************************************
prompt &&separator
prompt Checking DBMS_SQL, DBMS_OUTPUT and DBMS_LOCK...

col c1 noprint
BEGIN
   select NVL(count(1),0)  pkg_count
     into :pkg_count
     from dba_objects
    where object_type IN ('PACKAGE','PACKAGE BODY')
      and object_name IN ('DBMS_SQL','DBMS_OUTPUT','DBMS_LOCK')
      and owner='SYS'
      and status = 'INVALID';
   IF :pkg_count > 0
   THEN 
      :abort := :abort +1;
   END IF;
END;
/
SELECT  0 c1
       ,decode(:pkg_count,0,to_char(null),
         'ERROR: One or more of the following ORACLE packages is missing or invalid:')
  FROM dual
UNION
SELECT  1 c1
       ,decode(:pkg_count,0,to_char(null),
         'ERROR: DBMS_SQL, DBMS_LOCK, DBMS_OUTPUT')
  FROM dual
UNION
SELECT  2 c1
       ,decode(:pkg_count,0,
         'The packages are valid, continuing.',to_char(null))
  FROM dual;

prompt &&separator

REM *************************************************************************
REM this section will abort script if problem found above
REM
WHENEVER SQLERROR EXIT
BEGIN
   dbms_lock.sleep(5);
   IF :abort > 0
   THEN
      raise_application_error(-20001, 'cannot continue...exiting');
   END IF;
END;
/

REM *************************************************************************
prompt &&separator
prompt Creating q$instl package...
prompt

rem *** EXIT if cannot find package files for some reason

WHENEVER OSERROR EXIT SUCCESS

set feedback on
start &&source_dir\qinstl&&O7ver..wps
start &&source_dir\qinstl&&O7ver..wpb
set feedback off

WHENEVER SQLERROR CONTINUE
WHENEVER OSERROR CONTINUE

REM *************************************************************************
prompt &&separator
prompt Checking if Q installed...
prompt

rem driver_stopped used to signal that one QDBA connection may be 
rem a job_queue bgproc driver process not yet cleaned up
var driver_stopped NUMBER

DECLARE
   driver_running  BOOLEAN;
   driver_jobno    INTEGER;
BEGIN
   :driver_stopped :=0;
   --
   IF q$instl.Qver = q$instl.q_not_installed
   THEN
      dbms_output.put_line('Q is not installed on this database.');
   ELSE
      dbms_output.put_line('Q version installed: '||q$instl.Qver);
      --
      -- check the status of BGPROC.DRIVER
      q$instl.check_driver(driver_running,driver_jobno);
      IF driver_running
      THEN
         dbms_output.put_line
            ('Q Diagnostic Engine is running as job '||driver_jobno||
               '...stopping Diagnostic Engine.');
         q$instl.stop_driver;
         dbms_lock.sleep(5);
         q$instl.check_driver(driver_running,driver_jobno);
         IF driver_running
         THEN
            :check_return_code := :check_return_code +1;
            dbms_output.put_line('DISPLAY:'); 
            dbms_output.put_line
               ('ERROR: Failed to stop Q Diagnostic Engine');
            dbms_output.put_line('See documentation for instructions on manually stopping '||
               'the Diagnostic Engine');
            dbms_output.put_line('NODISPLAY:'); 
         ELSE
            dbms_output.put_line
               ('Q Diagnostic Engine has been stopped.');
            :driver_stopped := 1;
         END IF;
      ELSE
         dbms_output.put_line
            ('The Q Diagnostic Engine is not running.');
      END IF;
      --
   END IF;
END;
/
prompt &&separator

REM *************************************************************************
prompt &&separator
prompt Checking SYSTEM tablespace...
prompt
DECLARE
   totfree_kB  NUMBER;
   maxfree_kB  NUMBER;
   return_code INTEGER:=0;
   PROCEDURE maxfree_msg
      (segment_name_IN IN VARCHAR2
      ,next_extent_kb IN NUMBER)
   IS
   BEGIN
      dbms_output.put_line('DISPLAY:'); 
      dbms_output.put_line('ERROR: The segment '||segment_name_IN||' in SYSTEM tablespace ');
      dbms_output.put_line('has next extent size '||TO_CHAR(next_extent_kb)||' kB which exceeds the ');
      dbms_output.put_line('maximum free extent size of '||TO_CHAR(maxfree_kB)||' kB.');
      dbms_output.put_line('.');
      dbms_output.put_line('The segment could incur extent allocation failure.  This can be ');
      dbms_output.put_line('corrected by adding space to the SYSTEM tablespace or modifying');
      dbms_output.put_line('the segment storage parameters as follows: ');
      dbms_output.put_line('.');
      dbms_output.put_line('ALTER TABLE '||segment_name_IN||
                              ' STORAGE ( NEXT '||TO_CHAR(LEAST(512,maxfree_kb))||'K );');
      dbms_output.put_line('NODISPLAY:'); 
   END maxfree_msg;
   --
BEGIN
   totfree_kB := ROUND(q$instl.system_freespace/1024,2);
   maxfree_kb := ROUND(q$instl.system_maxfree/1024,2);
   IF  ( totfree_kB < 1100 AND q$instl.Qver = q$instl.q_not_installed )
      OR totfree_kB < 300
   THEN
      return_code := return_code +1;
      dbms_output.put_line('DISPLAY:'); 
      dbms_output.put_line('ERROR: SYSTEM tablespace free space low');
      dbms_output.put_line('The SYSTEM tablespace has only '||totfree_kB||' kB of free space.');
      dbms_output.put_line('Installing Q may exhaust SYSTEM free space due to catalog expansion.');
      dbms_output.put_line('More space can be allocated to the SYSTEM tablespace by adding a');
      dbms_output.put_line('datafile using the ALTER TABLESPACE command.');
      dbms_output.put_line('.');
      dbms_output.put_line('For example, to add 10 megabytes to SYSTEM (choose filename):');
      dbms_output.put_line('.');
      dbms_output.put_line('ALTER TABLESPACE SYSTEM ADD DATAFILE filename SIZE 10M;');
      dbms_output.put_line('.');
      dbms_output.put_line('NODISPLAY:'); 
   END IF;
   --
   IF   maxfree_kB < q$instl.seg_nextext('SYS','IDL_CHAR$','TABLE')/1024
   THEN
      maxfree_msg('SYS.IDL_CHAR$',TRUNC(q$instl.seg_nextext('SYS','IDL_CHAR$','TABLE')/1024) );
      return_code := return_code +1;
   END IF;
   --
   IF   maxfree_kB < q$instl.seg_nextext('SYS','IDL_SB4$','TABLE')/1024
   THEN
      maxfree_msg('SYS.IDL_SB4$',TRUNC(q$instl.seg_nextext('SYS','IDL_SB4$','TABLE')/1024) );
      return_code := return_code +1;
   END IF;
   --
   IF   maxfree_kB < q$instl.seg_nextext('SYS','IDL_UB1$','TABLE')/1024
   THEN
      maxfree_msg('SYS.IDL_UB1$',TRUNC(q$instl.seg_nextext('SYS','IDL_UB1$','TABLE')/1024) );
      return_code := return_code +1;
   END IF;
   --
   IF   maxfree_kB < q$instl.seg_nextext('SYS','IDL_UB2$','TABLE')/1024
   THEN
      maxfree_msg('SYS.IDL_UB2$',TRUNC(q$instl.seg_nextext('SYS','IDL_UB2$','TABLE')/1024) );
      return_code := return_code +1;
   END IF;
   --
   IF   maxfree_kB < q$instl.seg_nextext('SYS','SOURCE$','TABLE')/1024
    THEN
      maxfree_msg('SYS.SOURCE$',TRUNC(q$instl.seg_nextext('SYS','SOURCE$','TABLE')/1024) );
      return_code := return_code +1;
   END IF;
   --
   IF return_code = 0
   THEN 
      dbms_output.put_line('SYSTEM tablespace has sufficient space for Q installation.');
   END IF;
   --
   :check_return_code := :check_return_code + return_code;
END;
/
prompt &&separator

REM *************************************************************************
prompt &&separator
prompt Checking ORACLE required packages...
prompt
DECLARE
   temp_return_code  INTEGER:=0;
BEGIN
   q$instl.check_object('SYS','DBMS_SHARED_POOL','PACKAGE',temp_return_code);
   q$instl.check_object('SYS','DBMS_SHARED_POOL','PACKAGE BODY',temp_return_code);
   q$instl.check_object('SYS','DBMS_PIPE','PACKAGE',temp_return_code);
   q$instl.check_object('SYS','DBMS_PIPE','PACKAGE BODY',temp_return_code);
   q$instl.check_object('SYS','DBMS_LOCK','PACKAGE',temp_return_code);
   q$instl.check_object('SYS','DBMS_LOCK','PACKAGE BODY',temp_return_code);
   q$instl.check_object('SYS','DBMS_SQL','PACKAGE',temp_return_code);
   q$instl.check_object('SYS','DBMS_SQL','PACKAGE BODY',temp_return_code);
   q$instl.check_object('SYS','DBMS_SPACE','PACKAGE',temp_return_code);
   q$instl.check_object('SYS','DBMS_SPACE','PACKAGE BODY',temp_return_code);
   q$instl.check_object('SYS','DBMS_UTILITY','PACKAGE',temp_return_code);
   q$instl.check_object('SYS','DBMS_UTILITY','PACKAGE BODY',temp_return_code);
   q$instl.check_object('SYS','DBMS_SESSION','PACKAGE',temp_return_code);
   q$instl.check_object('SYS','DBMS_SESSION','PACKAGE BODY',temp_return_code);
   q$instl.check_object('SYS','DBMS_JOB','PACKAGE',temp_return_code);
   q$instl.check_object('SYS','DBMS_JOB','PACKAGE BODY',temp_return_code);
   q$instl.check_object('SYS','DBMS_IJOB','PACKAGE',temp_return_code);
   q$instl.check_object('SYS','DBMS_IJOB','PACKAGE BODY',temp_return_code);
   q$instl.check_object('SYS','DBMS_OUTPUT','PACKAGE',temp_return_code);
   q$instl.check_object('SYS','DBMS_OUTPUT','PACKAGE BODY',temp_return_code);
   --
   IF temp_return_code = 0 
   THEN
      dbms_output.put_line('All required ORACLE packages are VALID.');
   ELSE
      :check_return_code := :check_return_code + temp_return_code;
      dbms_output.put_line('DISPLAY:'); 
      dbms_output.put_line
         ('ERROR: Detected '||temp_return_code||' ERRORS with required packages.');
      dbms_output.put_line('.');
      dbms_output.put_line
         ('The Q Diagnostic Engine for ORACLE requires these built-in packages,');
      dbms_output.put_line
         ('please (re)install them before continuing with Q installation. ');
      dbms_output.put_line('.');
      dbms_output.put_line('INVALID DBMS packages can be re-created by executing the ');
      dbms_output.put_line('package creation scripts from SVRMGR when connected as SYS or ');
      dbms_output.put_line('INTERNAL (preferred).  These scripts are located in the ');
      dbms_output.put_line('<ORACLE_HOME>/rdbms/admin directory.  Each package has a pair ');
      dbms_output.put_line('of files to run named DBMS{name}.SQL and PRVT{name}.PLB. ');
      dbms_output.put_line('.');
      dbms_output.put_line('For instance to create the DBMS_SHARED_POOL package, connect to');
      dbms_output.put_line('SVRMGR as SYS and execute: ');
      dbms_output.put_line('.');
      dbms_output.put_line('SVRMGR> @<ORACLE_HOME>/rdbms/admin/dbmspool.sql');
      dbms_output.put_line('SVRMGR> @<ORACLE_HOME>/rdbms/admin/prvtpool.plb');
      --
      dbms_output.put_line('.');
      dbms_output.put_line('NODISPLAY:'); 
   END IF;
END;
/
prompt &&separator

REM *************************************************************************
prompt &&separator
prompt Checking ORACLE job queue...
prompt
DECLARE
   parm_value        VARCHAR2(200);
   job_procs         NUMBER;
   job_interval      NUMBER;
BEGIN
   dbms_output.put_line('JOB_QUEUE init.ora parameter values:');
   dbms_output.put_line('.');
   q$instl.get_initparm('job_queue_processes',parm_value);
   job_procs := to_number(parm_value);
   dbms_output.put_line('job_queue_processes = '||job_procs);
   --
   q$instl.get_initparm('job_queue_interval',parm_value);
   job_interval := to_number(parm_value);
   dbms_output.put_line('job_queue_interval = '||job_interval);
   --
   IF job_procs = 0
   THEN
      :check_return_code := :check_return_code +1; 
      dbms_output.put_line
         ('ERROR: Job Queue processes not running.');
   END IF;
   dbms_output.put_line('Recommended INIT.ORA settings:');
   dbms_output.put_line
      ('job_queue_processes = '||LEAST(GREATEST(job_procs+1,2),36) );
   dbms_output.put_line
      ('job_queue_interval = '||LEAST(job_interval,60) );
END;
/
prompt &&separator

REM *************************************************************************
prompt &&separator
prompt Checking timed_statistics...
prompt
DECLARE
   parm_value        VARCHAR2(200);
BEGIN
   q$instl.get_initparm('timed_statistics',parm_value);
   --
   dbms_output.put_line('DISPLAY:');
   dbms_output.put_line('TIMED_STATISTICS init.ora value set to '||parm_value);
   --
   IF parm_value != 'TRUE'
   THEN
      dbms_output.put_line('Savant recommends that this parameter be set to TRUE');
      dbms_output.put_line('for more effective diagnostic metrics.');
   END IF;
   dbms_output.put_line('NODISPLAY:');
END;
/
prompt &&separator

REM *************************************************************************
prompt &&separator
prompt Checking ORACLE shared_pool...
prompt

ALTER SYSTEM FLUSH SHARED_POOL;

DECLARE
   free_chunks       NUMBER;
   max_free          NUMBER;
   tot_free          NUMBER;
   shared_pool_size  NUMBER;
   shared_pool_min_alloc  NUMBER;
   shared_pool_reserved   NUMBER;
BEGIN
   dbms_output.put_line('SHARED POOL init.ora parameter values:');
   dbms_output.put_line('.');
   --
   q$instl.get_initparm('shared_pool_size',shared_pool_size);
   dbms_output.put_line('shared_pool_size = '||shared_pool_size);
   --
   q$instl.get_initparm('shared_pool_reserved_size',shared_pool_reserved);
   dbms_output.put_line('shared_pool_reserved_size = '||shared_pool_reserved);
   --
   q$instl.get_initparm('shared_pool_reserved_min_alloc',shared_pool_min_alloc);
   dbms_output.put_line('shared_pool_reserved_min_alloc = '||shared_pool_min_alloc);
   --
   q$instl.shared_pool_free(30,free_chunks,tot_free,max_free);
   dbms_output.put_line('Free Shared Pool chunks > 30kB  = '||free_chunks);
   dbms_output.put_line('Total free Shared Pool = '||tot_free);
   dbms_output.put_line('Max free chunk = '||max_free);
   --
   dbms_output.put_line('DISPLAY:'); 
   --
   IF tot_free < 1000 OR max_free < 50
   THEN
      dbms_output.put_line
         ('WARNING: Shared Pool memory is constrained. ');
   ELSIF free_chunks < 5
   THEN
      dbms_output.put_line
         ('WARNING: Shared Pool memory is fragmented. ');
   END IF;
   dbms_output.put_line('Recommended INIT.ORA settings:');
   dbms_output.put_line
      ('shared_pool_size = '||
      LTRIM(to_char(shared_pool_size+1000000,'99999999990')));
   dbms_output.put_line
      ('shared_reserved_size = '||
      LTRIM(to_char(GREATEST(shared_pool_reserved,1000000),'99999999990')));
   dbms_output.put_line
      ('shared_pool_reserved_min_alloc = '||
      LTRIM(to_char(GREATEST(shared_pool_min_alloc,10000),'99999999990')));
   --
   dbms_output.put_line('NODISPLAY:'); 
END;
/

REM *************************************************************************
prompt &&separator
prompt Checking for connected QDBA users...
prompt

BEGIN
      IF q$instl.qdba_connected > :driver_stopped 
      THEN
         dbms_output.put_line
         ('ERROR: There are '||q$instl.qdba_connected||' QDBA connections, '||
            'disconnect before installing Q.');
         :check_return_code := :check_return_code +1;
      ELSE
         dbms_output.put_line
            ('No QDBA users are currently connected.');
      END IF;
END;
/
prompt &&separator
prompt End of QCHKALL.SQL
