/* TABGRANT.SQL */

/* Lists all direct grants made against a specific table, view, or program,  */
/* segregated by the type of the grant recipient.  This report is a complete */
/* listing of all methods by which the object may be accessed.  Used to      */
/* research the potential impact of revokes against objects that have        */
/* multiple access paths that lead to it.                                    */

/* Input:  A user name and the name of a table, view, or program in the */
/*         schema */

/* Output Column Legend: */
/*   LEVEL indicates whether the permission on the object was issued to */
/*         PUBLIC, a role, or directly to a user */
/*   PERMISSION contains the name of the privilege granted against the object */

set pagesize 66
set verify off
set feedback off
column lvl format A8 Heading "Level"
column grantee format A27
column privilege format A27
column grantable format A4 heading "Gtbl"
ttitle left 'TABGRANT - Direct Grants made to '&2 skip 2
spool tabgrant.lst
set termout off
SELECT 'Role' lvl, t.grantee, t.privilege, t.grantable
FROM dba_tab_privs t
WHERE t.owner = UPPER('&1') AND t.table_name = UPPER('&2') AND
      t.grantee IN (SELECT role FROM dba_roles WHERE role = t.grantee)
UNION
SELECT 'User' lvl, t.grantee, t.privilege, t.grantable
FROM dba_tab_privs t
WHERE t.owner = UPPER('&1') AND t.table_name = UPPER('&2') AND
      t.grantee IN (SELECT username FROM dba_users WHERE username = t.grantee)
UNION
SELECT 'Public' lvl, t.grantee, t.privilege, t.grantable
FROM dba_tab_privs t
WHERE t.owner = UPPER('&1') AND t.table_name = UPPER('&2') AND
      t.grantee = 'PUBLIC'
ORDER BY 1, 2, 3;
spool off
set termout on
/* prompt Created tabgrant.lst for your viewing pleasure... */
exit
