rem $DBA/tablstat.sql
rem
rem Prints a report containing statistics (Number of Rows, Number of Bytes,
rem DB_BLOCK_SIZE, PCTFREE, Column name, Data type, Length, (optionally)
rem Percent of Distinct Columns, Indexed Columns, and Constraints) about a
rem specific table (&1) in a database
rem
rem Restrictions:
rem	1) Requires this script to be run from an account with DBA privileges.
rem	2) Requires that the DBMS_SQL package be executable (if the percentage
rem	   of the distinct columns is to be calculated).
rem	3) Requires the $DBA/tablecon.sql SQL script to exist.
rem
rem Last Change 09/15/97 by Brian Lomasky to move table info into PL/SQL,
rem					  reformat headings.
rem Last Change 04/10/97 by Brian Lomasky to add bitmap indexes
rem Last Change 10/28/96 by Brian Lomasky
rem
set pagesize 0
set head off
set term off
set feedback off
set verify off
def cr=chr(10)
column dbb new_value blksiz
column own new_value ownr
column fre new_value free
column cnt new_value kount
column blx new_value blox
select value dbb from v$parameter where name = 'db_block_size';
select owner own, pct_free fre from dba_tables where table_name = '&&1';
select count(*) cnt from &ownr..&&1;
select count(distinct(substr(rowid,15,4)||substr(rowid,1,8))) blx
	from &ownr..&&1;
drop table tablstat_temp;
set term on
create table tablstat_temp(
	lineno		number,
	ttxt		varchar2(80)
	);
declare
	cursor tbl is
		select	column_name,
			data_type,
			data_length,
			data_precision,
			data_scale
		from sys.dba_tab_columns
		where table_name = '&&1'
		order by column_id;
	cursor ind is
		select	index_name,
			owner,
			uniqueness
		from sys.dba_indexes
		where table_name = '&&1';

	lv_column_name		sys.dba_tab_columns.column_name%TYPE;
	lv_data_type		sys.dba_tab_columns.data_type%TYPE;
	lv_data_length		sys.dba_tab_columns.data_length%TYPE;
	lv_data_precision	sys.dba_tab_columns.data_precision%TYPE;
	lv_data_scale		sys.dba_tab_columns.data_scale%TYPE;
	lv_index_name		sys.dba_indexes.index_name%TYPE;
	lv_owner		sys.dba_indexes.owner%TYPE;
	lv_uniqueness		sys.dba_indexes.uniqueness%TYPE;
	column_name		char(30);
	data_type		char(9);
	the_length		char(7);
	my_cursor		number;
	dummy			number;
	pct_dist		number;
	linen			number;
	txt			varchar2(80);

	/* Function to calculate the Pct Distinctness of a given column */
	function pct_distinct(a_col IN varchar2) return char is
	begin
		if &kount = 0 then
			return '       0';
		else
			my_cursor := dbms_sql.open_cursor;
			dbms_sql.parse(my_cursor,
				'select count(distinct ' || a_col || ')' ||
				' from &ownr..&&1', dbms_sql.v7);
			dbms_sql.define_column(my_cursor, 1, pct_dist);
			dummy := dbms_sql.execute(my_cursor);
			if dbms_sql.fetch_rows(my_cursor) > 0 then
				dbms_sql.column_value(my_cursor, 1, pct_dist);
			else
				pct_dist := 0;
			end if;
			dbms_sql.close_cursor(my_cursor);
			return lpad(round(pct_dist * 100 / &kount), 7);
		end if;
	end pct_distinct;
begin
	linen := 0;
	txt := ' ';
	linen := linen + 1;
	insert into tablstat_temp values (linen, txt);
	if '&&3' = 'Y' then
		txt := '                                                    ' ||
			'Percent';
		linen := linen + 1;
		insert into tablstat_temp values (linen, txt);
		txt := '                                                    ' ||
			'distinct';
		linen := linen + 1;
		insert into tablstat_temp values (linen, txt);
		txt := '   Column                         Datatype   Length ' ||
			' values';
		linen := linen + 1;
		insert into tablstat_temp values (linen, txt);
		txt := '   ------------------------------ --------- ------- ' ||
			'--------';
	else
		txt := '   Column                         Datatype   Length';
		linen := linen + 1;
		insert into tablstat_temp values (linen, txt);
		txt := '   ------------------------------ --------- -------';
	end if;
	linen := linen + 1;
	insert into tablstat_temp values (linen, txt);
	open tbl;
	loop
		fetch tbl into
			lv_column_name,
			lv_data_type,
			lv_data_length,
			lv_data_precision,
			lv_data_scale;
		exit when tbl%notfound;
		column_name := lv_column_name;
		data_type := lv_data_type;
		if lv_data_type = 'NUMBER' and lv_data_precision is not null or
			lv_data_type = 'FLOAT'
		then
			if nvl(lv_data_scale, 0) = 0 then
				the_length := lpad('(' ||
					to_char(lv_data_precision) || ')', 7);
			else
				the_length := lpad('(' ||
					to_char(lv_data_precision) || ',' ||
					to_char(lv_data_scale) || ')', 7);
			end if;
		else
			the_length := to_char(lv_data_length, '999999');
		end if;
		if lv_data_type = 'LONG' then
			txt := '   ' || column_name || ' ' || data_type ||
				' ' || the_length || '    n/a';
		else
			if '&&3' = 'N' then
				txt := '   ' || column_name || ' ' ||
					data_type || ' ' || the_length;
			else
				txt := '   ' || column_name || ' ' ||
					data_type || ' ' || the_length ||
					pct_distinct(lv_column_name) || '%';
			end if;
		end if;
		linen := linen + 1;
		insert into tablstat_temp values (linen, txt);
	end loop;
	close tbl;
	open ind;
	loop
		fetch ind into lv_index_name, lv_owner, lv_uniqueness;
		exit when ind%notfound;
		declare
			lv_col_name    sys.dba_ind_columns.column_name%TYPE;
			lv_col_pos     sys.dba_ind_columns.column_position%TYPE;
			cursor ind_col is
				select column_name, column_position
				from sys.dba_ind_columns
				where index_name = lv_index_name and
					index_owner = lv_owner
				order by column_position;
		begin
			open ind_col;
			loop
				fetch ind_col into
					lv_col_name,
					lv_col_pos;
				exit when ind_col%notfound;
				if lv_col_pos = 1 then
					if lv_uniqueness = 'UNIQUE' then
						txt := '     Unique Index ' ||
							rtrim(lv_index_name) ||
							' on ' ||
							rtrim(lv_col_name);
					elsif lv_uniqueness = 'BITMAP' then
						txt := '     Bitmap Index ' ||
							rtrim(lv_index_name) ||
							' on ' ||
							rtrim(lv_col_name);
					else
						txt := '     Index ' ||
							rtrim(lv_index_name) ||
							' on ' ||
							rtrim(lv_col_name);
					end if;
				else
					if lv_uniqueness = 'UNIQUE' then
						txt := '            '||
							rpad(' ',length(rtrim(
							lv_index_name))+10) ||
							rtrim(lv_col_name);
					elsif lv_uniqueness = 'BITMAP' then
						txt := '            '||
							rpad(' ',length(rtrim(
							lv_index_name))+10) ||
							rtrim(lv_col_name);
					else
						txt := '     '||
							rpad(' ',length(rtrim(
							lv_index_name))+10) ||
							rtrim(lv_col_name);
					end if;
				end if;
				linen := linen + 1;
				insert into tablstat_temp values (linen, txt);
			end loop;
			close ind_col;
			commit;
		end;
	end loop;
	close ind;
end;
/
set term off
spool tablstat_tmp.sql
select 'set linesize 80' from dual;
select 'set pagesize 0' from dual;
select 'set head off' from dual;
select 'set term off' from dual;
select 'column byt format 999,999,999' from dual;
select 'column bsz format 9999' from dual;
select 'column fr format 999' from dual;
select 'spool tablstat.lis' from dual;
select 'select ''&&2 table &ownr..&&1 contains ''||' from dual;
select '&kount||'' rows in'' from dual;' from dual;
select 'select '||&blox * &blksiz||' byt, ''bytes, DB_BLOCK_SIZE of'','||
	&blksiz||' bsz,' from dual;
select '''PCTFREE of '','||&free||' fr from dual;' from dual;
select 'select ttxt from tablstat_temp order by lineno;' from dual;
select '@$DBA/tablecon "&&1"' from dual;
select 'spool off' from dual;
spool off
@tablstat_tmp
set term on
drop table tablstat_temp;
exit
